@@ -25,28 +25,20 @@
  */
 
 #include "config.h"
-#include <time.h>
+
+#include "gdkcolor.h"
 
 #include "gdkscreen.h"
-#include "gdkcolor.h"
 #include "gdkinternals.h"
 
+#include <time.h>
 
 /**
- * gdk_colormap_get_visual:
- * @colormap: a #GdkColormap.
- * 
- * Returns the visual for which a given colormap was created.
- * 
- * Return value: the visual of the colormap.
- **/
-GdkVisual *
-gdk_colormap_get_visual (GdkColormap *colormap)
-{
-  g_return_val_if_fail (GDK_IS_COLORMAP (colormap), NULL);
+ * SECTION:colors
+ * @Short_description: Manipulation of colors
+ * @Title: Colors
+ */
 
-  return colormap->visual;
-}
 
 /**
  * gdk_color_copy:
@@ -84,39 +76,6 @@ gdk_color_free (GdkColor *color)
   g_slice_free (GdkColor, color);
 }
 
-/********************
- * Color allocation *
- ********************/
-
-/**
- * gdk_colormap_alloc_color:
- * @colormap: a #GdkColormap.
- * @color: the color to allocate. On return the
- *    <structfield>pixel</structfield> field will be
- *    filled in if allocation succeeds.
- * @writeable: this parameter has no effect, and it's here for mere
- *   compatibility.
- * @best_match: If %TRUE, GDK will attempt to do matching against
- *    existing colors if the color cannot be allocated as requested.
- *
- * Allocates a single color from a colormap.
- *
- * Return value: %TRUE if the allocation succeeded.
- **/
-gboolean
-gdk_colormap_alloc_color (GdkColormap *colormap,
-			  GdkColor    *color,
-			  gboolean     writeable,
-			  gboolean     best_match)
-{
-  gboolean success;
-
-  gdk_colormap_alloc_colors (colormap, color, 1, writeable, best_match,
-			     &success);
-
-  return success;
-}
-
 /**
  * gdk_color_hash:
  * @colora: a #GdkColor.
@@ -156,17 +115,9 @@ gdk_color_equal (const GdkColor *colora,
 	  (colora->blue == colorb->blue));
 }
 
-GType
-gdk_color_get_type (void)
-{
-  static GType our_type = 0;
-  
-  if (our_type == 0)
-    our_type = g_boxed_type_register_static (g_intern_static_string ("GdkColor"),
-					     (GBoxedCopyFunc)gdk_color_copy,
-					     (GBoxedFreeFunc)gdk_color_free);
-  return our_type;
-}
+G_DEFINE_BOXED_TYPE (GdkColor, gdk_color,
+                     gdk_color_copy,
+                     gdk_color_free)
 
 /**
  * gdk_color_parse:
@@ -176,15 +127,13 @@ gdk_color_get_type (void)
  * Parses a textual specification of a color and fill in the
  * <structfield>red</structfield>, <structfield>green</structfield>,
  * and <structfield>blue</structfield> fields of a #GdkColor
- * structure. The color is <emphasis>not</emphasis> allocated, you
- * must call gdk_colormap_alloc_color() yourself. The string can
- * either one of a large set of standard names. (Taken from the X11
- * <filename>rgb.txt</filename> file), or it can be a hex value in the
- * form '&num;rgb' '&num;rrggbb' '&num;rrrgggbbb' or
- * '&num;rrrrggggbbbb' where 'r', 'g' and 'b' are hex digits of the
- * red, green, and blue components of the color, respectively. (White
- * in the four forms is '&num;fff' '&num;ffffff' '&num;fffffffff' and
- * '&num;ffffffffffff')
+ * structure. The string can either one of a large set of standard
+ * names. (Taken from the X11 <filename>rgb.txt</filename> file), or
+ * it can be a hex value in the form '&num;rgb' '&num;rrggbb'
+ * '&num;rrrgggbbb' or '&num;rrrrggggbbbb' where 'r', 'g' and 'b' are
+ * hex digits of the red, green, and blue components of the color,
+ * respectively. (White in the four forms is '&num;fff' '&num;ffffff'
+ * '&num;fffffffff' and '&num;ffffffffffff')
  * 
  * Return value: %TRUE if the parsing succeeded.
  **/
@@ -232,17 +181,3 @@ gdk_color_to_string (const GdkColor *color)
 
   return pango_color_to_string (&pango_color);
 }
-
-/**
- * gdk_colormap_get_system:
- * 
- * Gets the system's default colormap for the default screen. (See
- * gdk_colormap_get_system_for_screen ())
- * 
- * Return value: the default colormap.
- **/
-GdkColormap*
-gdk_colormap_get_system (void)
-{
-  return gdk_screen_get_system_colormap (gdk_screen_get_default ());
-}
