@@ -24,19 +24,13 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#include <config.h>
-#include <math.h>
-#include "gdk.h"
+#include "config.h"
+
 #include "gdkwindow.h"
 #include "gdkinternals.h"
 #include "gdkwindowimpl.h"
-#include "gdkpixmap.h"
-#include "gdkdrawable.h"
-#include "gdktypes.h"
-#include "gdkscreen.h"
-#include "gdkcolor.h"
-#include "gdkcursor.h"
 
+#include <math.h>
 
 /* LIMITATIONS:
  *
@@ -53,11 +47,8 @@ struct _GdkOffscreenWindow
   GdkDrawable parent_instance;
 
   GdkWindow *wrapper;
-  GdkCursor *cursor;
-  GdkColormap *colormap;
-  GdkScreen *screen;
 
-  GdkPixmap *pixmap;
+  cairo_surface_t *surface;
   GdkWindow *embedder;
 };
 
@@ -88,12 +79,8 @@ gdk_offscreen_window_finalize (GObject *object)
 {
   GdkOffscreenWindow *offscreen = GDK_OFFSCREEN_WINDOW (object);
 
-  if (offscreen->cursor)
-    gdk_cursor_unref (offscreen->cursor);
-
-  offscreen->cursor = NULL;
-
-  g_object_unref (offscreen->pixmap);
+  if (offscreen->surface)
+    cairo_surface_destroy (offscreen->surface);
 
   G_OBJECT_CLASS (gdk_offscreen_window_parent_class)->finalize (object);
 }
@@ -117,9 +104,22 @@ gdk_offscreen_window_destroy (GdkWindow *window,
   
   if (!recursing)
     gdk_offscreen_window_hide (window);
+}
 
-  g_object_unref (offscreen->colormap);
-  offscreen->colormap = NULL;
+static cairo_surface_t *
+get_surface (GdkOffscreenWindow *offscreen)
+{
+  if (! offscreen->surface)
+    {
+      GdkWindowObject *private = (GdkWindowObject *) offscreen->wrapper;
+
+      g_signal_emit_by_name (private, "create-surface",
+                             private->width,
+                             private->height,
+                             &offscreen->surface);
+    }
+
+  return offscreen->surface;
 }
 
 static gboolean
@@ -145,74 +145,35 @@ gdk_offscreen_window_ref_cairo_surface (GdkDrawable *drawable)
 {
   GdkOffscreenWindow *offscreen = GDK_OFFSCREEN_WINDOW (drawable);
 
-  return _gdk_drawable_ref_cairo_surface (offscreen->pixmap);
+  return cairo_surface_reference (get_surface (offscreen));
 }
 
-static GdkColormap*
-gdk_offscreen_window_get_colormap (GdkDrawable *drawable)
+cairo_surface_t *
+_gdk_offscreen_window_create_surface (GdkWindow *offscreen,
+                                      gint       width,
+                                      gint       height)
 {
-  GdkOffscreenWindow *offscreen = GDK_OFFSCREEN_WINDOW (drawable);
+  GdkWindowObject *private = (GdkWindowObject *) offscreen;
+  cairo_surface_t *similar;
+  cairo_surface_t *surface;
 
-  return offscreen->colormap;
-}
+  g_return_val_if_fail (GDK_IS_OFFSCREEN_WINDOW (private->impl), NULL);
 
-static void
-gdk_offscreen_window_set_colormap (GdkDrawable *drawable,
-				   GdkColormap*colormap)
-{
-  GdkOffscreenWindow *offscreen = GDK_OFFSCREEN_WINDOW (drawable);
+  similar = _gdk_drawable_ref_cairo_surface ((GdkWindow *)private->parent);
 
-  if (colormap && GDK_WINDOW_DESTROYED (offscreen->wrapper))
-    return;
+  surface = cairo_surface_create_similar (similar,
+                                          /* FIXME: use visual */
+                                          CAIRO_CONTENT_COLOR,
+                                          width,
+                                          height);
 
-  if (offscreen->colormap == colormap)
-    return;
+  cairo_surface_destroy (similar);
 
-  if (offscreen->colormap)
-    g_object_unref (offscreen->colormap);
-
-  offscreen->colormap = colormap;
-  if (offscreen->colormap)
-    g_object_ref (offscreen->colormap);
-}
-
-
-static gint
-gdk_offscreen_window_get_depth (GdkDrawable *drawable)
-{
-  GdkOffscreenWindow *offscreen = GDK_OFFSCREEN_WINDOW (drawable);
-
-  return gdk_drawable_get_depth (offscreen->wrapper);
-}
-
-static GdkDrawable *
-gdk_offscreen_window_get_source_drawable (GdkDrawable  *drawable)
-{
-  GdkOffscreenWindow *offscreen = GDK_OFFSCREEN_WINDOW (drawable);
-
-  return _gdk_drawable_get_source_drawable (offscreen->pixmap);
-}
-
-static GdkScreen*
-gdk_offscreen_window_get_screen (GdkDrawable *drawable)
-{
-  GdkOffscreenWindow *offscreen = GDK_OFFSCREEN_WINDOW (drawable);
-
-  return offscreen->screen;
-}
-
-static GdkVisual*
-gdk_offscreen_window_get_visual (GdkDrawable    *drawable)
-{
-  GdkOffscreenWindow *offscreen = GDK_OFFSCREEN_WINDOW (drawable);
-
-  return gdk_drawable_get_visual (offscreen->wrapper);
+  return surface;
 }
 
 void
 _gdk_offscreen_window_new (GdkWindow     *window,
-			   GdkScreen     *screen,
-			   GdkVisual     *visual,
 			   GdkWindowAttr *attributes,
 			   gint           attributes_mask)
 {
@@ -232,27 +193,6 @@ _gdk_offscreen_window_new (GdkWindow     *window,
   private->impl = g_object_new (GDK_TYPE_OFFSCREEN_WINDOW, NULL);
   offscreen = GDK_OFFSCREEN_WINDOW (private->impl);
   offscreen->wrapper = window;
-
-  offscreen->screen = screen;
-
-  if (attributes_mask & GDK_WA_COLORMAP)
-    offscreen->colormap = g_object_ref (attributes->colormap);
-  else
-    {
-      if (gdk_screen_get_system_visual (screen) == visual)
-	{
-	  offscreen->colormap = gdk_screen_get_system_colormap (screen);
-	  g_object_ref (offscreen->colormap);
-	}
-      else
-	offscreen->colormap = gdk_colormap_new (visual, FALSE);
-    }
-
-  offscreen->pixmap = gdk_pixmap_new ((GdkDrawable *)private->parent,
-				      private->width,
-				      private->height,
-				      private->depth);
-  gdk_drawable_set_colormap (offscreen->pixmap, offscreen->colormap);
 }
 
 static gboolean
@@ -368,41 +308,6 @@ gdk_offscreen_window_get_root_coords (GdkWindow *window,
   return TRUE;
 }
 
-static gint
-gdk_offscreen_window_get_deskrelative_origin (GdkWindow *window,
-					      gint      *x,
-					      gint      *y)
-{
-  GdkWindowObject *private = GDK_WINDOW_OBJECT (window);
-  GdkOffscreenWindow *offscreen;
-  int tmpx, tmpy;
-
-  tmpx = 0;
-  tmpy = 0;
-
-  offscreen = GDK_OFFSCREEN_WINDOW (private->impl);
-  if (offscreen->embedder)
-    {
-      double dx, dy;
-      gdk_window_get_deskrelative_origin (offscreen->embedder,
-					  &tmpx, &tmpy);
-
-      to_embedder (window,
-		   0, 0,
-		   &dx, &dy);
-      tmpx = floor (tmpx + dx + 0.5);
-      tmpy = floor (tmpy + dy + 0.5);
-    }
-
-
-  if (x)
-    *x = tmpx;
-  if (y)
-    *y = tmpy;
-
-  return TRUE;
-}
-
 static gboolean
 gdk_offscreen_window_get_device_state (GdkWindow       *window,
                                        GdkDevice       *device,
@@ -441,19 +346,17 @@ gdk_offscreen_window_get_device_state (GdkWindow       *window,
 }
 
 /**
- * gdk_offscreen_window_get_pixmap:
+ * gdk_offscreen_window_get_surface:
  * @window: a #GdkWindow
  *
- * Gets the offscreen pixmap that an offscreen window renders into.
+ * Gets the offscreen surface that an offscreen window renders into.
  * If you need to keep this around over window resizes, you need to
  * add a reference to it.
  *
- * Returns: The offscreen pixmap, or %NULL if not offscreen
- *
- * Since: 2.18
+ * Returns: The offscreen surface, or %NULL if not offscreen
  */
-GdkPixmap *
-gdk_offscreen_window_get_pixmap (GdkWindow *window)
+cairo_surface_t *
+gdk_offscreen_window_get_surface (GdkWindow *window)
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
   GdkOffscreenWindow *offscreen;
@@ -464,7 +367,8 @@ gdk_offscreen_window_get_pixmap (GdkWindow *window)
     return NULL;
 
   offscreen = GDK_OFFSCREEN_WINDOW (private->impl);
-  return offscreen->pixmap;
+
+  return get_surface (offscreen);
 }
 
 static void
@@ -492,7 +396,6 @@ gdk_offscreen_window_move_resize_internal (GdkWindow *window,
   GdkWindowObject *private = (GdkWindowObject *)window;
   GdkOffscreenWindow *offscreen;
   gint dx, dy, dw, dh;
-  GdkPixmap *old_pixmap;
 
   offscreen = GDK_OFFSCREEN_WINDOW (private->impl);
 
@@ -515,21 +418,26 @@ gdk_offscreen_window_move_resize_internal (GdkWindow *window,
   if (private->width != width ||
       private->height != height)
     {
-      cairo_t *cr;
-
       private->width = width;
       private->height = height;
 
-      old_pixmap = offscreen->pixmap;
-      offscreen->pixmap = gdk_pixmap_new (GDK_DRAWABLE (old_pixmap),
-					  width,
-					  height,
-					  private->depth);
+      if (offscreen->surface)
+        {
+          cairo_surface_t *old_surface;
+          cairo_t *cr;
 
-      cr = gdk_cairo_create (offscreen->pixmap);
-      gdk_cairo_set_source_pixmap (cr, old_pixmap, 0, 0);
-      cairo_paint (cr);
-      cairo_destroy (cr);
+          old_surface = offscreen->surface;
+          offscreen->surface = NULL;
+
+          offscreen->surface = get_surface (offscreen);
+
+          cr = cairo_create (offscreen->surface);
+          cairo_set_source_surface (cr, old_surface, 0, 0);
+          cairo_paint (cr);
+          cairo_destroy (cr);
+
+          cairo_surface_destroy (old_surface);
+        }
     }
 
   if (GDK_WINDOW_IS_MAPPED (private))
@@ -575,9 +483,9 @@ gdk_offscreen_window_show (GdkWindow *window,
 			   gboolean already_mapped)
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
+  GdkRectangle area = { 0, 0, private->width, private->height };
 
-  gdk_window_clear_area_e (window, 0, 0,
-			   private->width, private->height);
+  gdk_window_invalidate_rect (window, &area, FALSE);
 }
 
 
@@ -594,7 +502,7 @@ gdk_offscreen_window_hide (GdkWindow *window)
   offscreen = GDK_OFFSCREEN_WINDOW (private->impl);
 
   /* May need to break grabs on children */
-  display = gdk_drawable_get_display (window);
+  display = gdk_window_get_display (window);
 
   /* TODO: This needs updating to the new grab world */
 #if 0
@@ -634,46 +542,8 @@ gdk_offscreen_window_set_events (GdkWindow       *window,
 
 static void
 gdk_offscreen_window_set_background (GdkWindow      *window,
-				     const GdkColor *color)
+				     cairo_pattern_t *pattern)
 {
-  GdkWindowObject *private = (GdkWindowObject *)window;
-
-  private->bg_color = *color;
-
-  if (private->bg_pixmap &&
-      private->bg_pixmap != GDK_PARENT_RELATIVE_BG &&
-      private->bg_pixmap != GDK_NO_BG)
-    g_object_unref (private->bg_pixmap);
-
-  private->bg_pixmap = NULL;
-}
-
-static void
-gdk_offscreen_window_set_back_pixmap (GdkWindow *window,
-				      GdkPixmap *pixmap)
-{
-  GdkWindowObject *private = (GdkWindowObject *)window;
-
-  if (pixmap &&
-      private->bg_pixmap != GDK_PARENT_RELATIVE_BG &&
-      private->bg_pixmap != GDK_NO_BG &&
-      !gdk_drawable_get_colormap (pixmap))
-    {
-      g_warning ("gdk_window_set_back_pixmap(): pixmap must have a colormap");
-      return;
-    }
-
-  if (private->bg_pixmap &&
-      private->bg_pixmap != GDK_PARENT_RELATIVE_BG &&
-      private->bg_pixmap != GDK_NO_BG)
-    g_object_unref (private->bg_pixmap);
-
-  private->bg_pixmap = pixmap;
-
-  if (pixmap &&
-      private->bg_pixmap != GDK_PARENT_RELATIVE_BG &&
-      private->bg_pixmap != GDK_NO_BG)
-    g_object_ref (pixmap);
 }
 
 static void
@@ -739,35 +609,48 @@ gdk_offscreen_window_translate (GdkWindow      *window,
                                 gint            dx,
                                 gint            dy)
 {
-  cairo_surface_t *surface;
-  cairo_t *cr;
+  GdkOffscreenWindow *offscreen = GDK_OFFSCREEN_WINDOW (((GdkWindowObject *) window)->impl);
 
-  /* Can't use gdk_cairo_create here due to clipping */
-  surface = _gdk_drawable_ref_cairo_surface (window);
-  cr = cairo_create (surface);
-  cairo_surface_destroy (surface);
+  if (offscreen->surface)
+    {
+      cairo_t *cr;
 
-  area = cairo_region_copy (area);
+      cr = cairo_create (offscreen->surface);
 
-  gdk_cairo_region (cr, area);
-  cairo_clip (cr);
-  
-  /* NB: This is a self-copy and Cairo doesn't support that yet.
-   * So we do a litle trick.
-   */
-  cairo_push_group (cr);
+      area = cairo_region_copy (area);
 
-  gdk_cairo_set_source_pixmap (cr, window, dx, dy);
-  cairo_paint (cr);
+      gdk_cairo_region (cr, area);
+      cairo_clip (cr);
 
-  cairo_pop_group_to_source (cr);
-  cairo_paint (cr);
+      /* NB: This is a self-copy and Cairo doesn't support that yet.
+       * So we do a litle trick.
+       */
+      cairo_push_group (cr);
 
-  cairo_destroy (cr);
+      cairo_set_source_surface (cr, offscreen->surface, dx, dy);
+      cairo_paint (cr);
+
+      cairo_pop_group_to_source (cr);
+      cairo_paint (cr);
+
+      cairo_destroy (cr);
+    }
 
   _gdk_window_add_damage (window, area);
 }
 
+static cairo_surface_t *
+gdk_offscreen_window_resize_cairo_surface (GdkWindow       *window,
+                                           cairo_surface_t *surface,
+                                           gint             width,
+                                           gint             height)
+{
+  /* No-op.  The surface gets resized in
+   * gdk_offscreen_window_move_resize_internal().
+   */
+  return surface;
+}
+
 /**
  * gdk_offscreen_window_set_embedder:
  * @window: a #GdkWindow
@@ -847,12 +730,6 @@ gdk_offscreen_window_class_init (GdkOffscreenWindowClass *klass)
   object_class->finalize = gdk_offscreen_window_finalize;
 
   drawable_class->ref_cairo_surface = gdk_offscreen_window_ref_cairo_surface;
-  drawable_class->set_colormap = gdk_offscreen_window_set_colormap;
-  drawable_class->get_colormap = gdk_offscreen_window_get_colormap;
-  drawable_class->get_depth = gdk_offscreen_window_get_depth;
-  drawable_class->get_screen = gdk_offscreen_window_get_screen;
-  drawable_class->get_visual = gdk_offscreen_window_get_visual;
-  drawable_class->get_source_drawable = gdk_offscreen_window_get_source_drawable;
 }
 
 static void
@@ -865,7 +742,6 @@ gdk_offscreen_window_impl_iface_init (GdkWindowImplIface *iface)
   iface->lower = gdk_offscreen_window_lower;
   iface->move_resize = gdk_offscreen_window_move_resize;
   iface->set_background = gdk_offscreen_window_set_background;
-  iface->set_back_pixmap = gdk_offscreen_window_set_back_pixmap;
   iface->get_events = gdk_offscreen_window_get_events;
   iface->set_events = gdk_offscreen_window_set_events;
   iface->reparent = gdk_offscreen_window_reparent;
@@ -876,7 +752,7 @@ gdk_offscreen_window_impl_iface_init (GdkWindowImplIface *iface)
   iface->queue_antiexpose = gdk_offscreen_window_queue_antiexpose;
   iface->translate = gdk_offscreen_window_translate;
   iface->get_root_coords = gdk_offscreen_window_get_root_coords;
-  iface->get_deskrelative_origin = gdk_offscreen_window_get_deskrelative_origin;
   iface->get_device_state = gdk_offscreen_window_get_device_state;
   iface->destroy = gdk_offscreen_window_destroy;
+  iface->resize_cairo_surface = gdk_offscreen_window_resize_cairo_surface;
 }
