@@ -65,7 +65,6 @@ typedef enum
  *  screen, and is created by the window system
  * @GDK_WINDOW_TOPLEVEL: toplevel window (used to implement #GtkWindow)
  * @GDK_WINDOW_CHILD: child window (used to implement e.g. #GtkEntry)
- * @GDK_WINDOW_DIALOG: useless/deprecated compatibility type
  * @GDK_WINDOW_TEMP: override redirect temporary window (used to implement
  *  #GtkMenu)
  * @GDK_WINDOW_FOREIGN: foreign window (see gdk_window_foreign_new())
@@ -79,7 +78,6 @@ typedef enum
   GDK_WINDOW_ROOT,
   GDK_WINDOW_TOPLEVEL,
   GDK_WINDOW_CHILD,
-  GDK_WINDOW_DIALOG,
   GDK_WINDOW_TEMP,
   GDK_WINDOW_FOREIGN,
   GDK_WINDOW_OFFSCREEN
@@ -91,7 +89,6 @@ typedef enum
  * @GDK_WA_X: Honor the X coordinate field
  * @GDK_WA_Y: Honor the Y coordinate field
  * @GDK_WA_CURSOR: Honor the cursor field
- * @GDK_WA_COLORMAP: Honor the colormap field
  * @GDK_WA_VISUAL: Honor the visual field
  * @GDK_WA_WMCLASS: Honor the wmclass_class and wmclass_name fields
  * @GDK_WA_NOREDIR: Honor the override_redirect field
@@ -110,11 +107,10 @@ typedef enum
   GDK_WA_X	   = 1 << 2,
   GDK_WA_Y	   = 1 << 3,
   GDK_WA_CURSOR	   = 1 << 4,
-  GDK_WA_COLORMAP  = 1 << 5,
-  GDK_WA_VISUAL	   = 1 << 6,
-  GDK_WA_WMCLASS   = 1 << 7,
-  GDK_WA_NOREDIR   = 1 << 8,
-  GDK_WA_TYPE_HINT = 1 << 9
+  GDK_WA_VISUAL	   = 1 << 5,
+  GDK_WA_WMCLASS   = 1 << 6,
+  GDK_WA_NOREDIR   = 1 << 7,
+  GDK_WA_TYPE_HINT = 1 << 8
 } GdkWindowAttributesType;
 
 /* Size restriction enumeration.
@@ -334,7 +330,6 @@ typedef enum
  * @wclass: #GDK_INPUT_OUTPUT (normal window) or #GDK_INPUT_ONLY (invisible
  *  window that receives events)
  * @visual: #GdkVisual for window
- * @colormap: #GdkColormap for window
  * @window_type: type of window
  * @cursor: cursor for the window (see gdk_window_set_cursor())
  * @wmclass_name: don't use (see gtk_window_set_wmclass())
@@ -353,7 +348,6 @@ struct _GdkWindowAttr
   gint height;
   GdkWindowClass wclass;
   GdkVisual *visual;
-  GdkColormap *colormap;
   GdkWindowType window_type;
   GdkCursor *cursor;
   gchar *wmclass_name;
@@ -495,6 +489,35 @@ typedef struct _GdkWindowObjectClass GdkWindowObjectClass;
 struct _GdkWindowObjectClass
 {
   GdkDrawableClass parent_class;
+
+  GdkWindow       * (* pick_embedded_child) (GdkWindow *window,
+                                             gdouble    x,
+                                             gdouble    y);
+
+  /*  the following 3 signals will only be emitted by offscreen windows */
+  void              (* to_embedder)         (GdkWindow *window,
+                                             gdouble    offscreen_x,
+                                             gdouble    offscreen_y,
+                                             gdouble   *embedder_x,
+                                             gdouble   *embedder_y);
+  void              (* from_embedder)       (GdkWindow *window,
+                                             gdouble    embedder_x,
+                                             gdouble    embedder_y,
+                                             gdouble   *offscreen_x,
+                                             gdouble   *offscreen_y);
+  cairo_surface_t * (* create_surface)      (GdkWindow *window,
+                                             gint       width,
+                                             gint       height);
+
+  /* Padding for future expansion */
+  void (*_gdk_reserved1) (void);
+  void (*_gdk_reserved2) (void);
+  void (*_gdk_reserved3) (void);
+  void (*_gdk_reserved4) (void);
+  void (*_gdk_reserved5) (void);
+  void (*_gdk_reserved6) (void);
+  void (*_gdk_reserved7) (void);
+  void (*_gdk_reserved8) (void);
 };
 
 /* Windows
@@ -507,6 +530,9 @@ void          gdk_window_destroy               (GdkWindow     *window);
 GdkWindowType gdk_window_get_window_type       (GdkWindow     *window);
 gboolean      gdk_window_is_destroyed          (GdkWindow     *window);
 
+GdkVisual *   gdk_window_get_visual            (GdkWindow     *window);
+GdkScreen *   gdk_window_get_screen            (GdkWindow     *window);
+GdkDisplay *  gdk_window_get_display           (GdkWindow     *window);
 #ifndef GDK_MULTIDEVICE_SAFE
 GdkWindow*    gdk_window_at_pointer            (gint          *win_x,
                                                 gint          *win_y);
@@ -530,17 +556,6 @@ void          gdk_window_reparent              (GdkWindow     *window,
                                                 GdkWindow     *new_parent,
                                                 gint           x,
                                                 gint           y);
-void          gdk_window_clear                 (GdkWindow     *window);
-void          gdk_window_clear_area            (GdkWindow     *window,
-                                                gint           x,
-                                                gint           y,
-                                                gint           width,
-                                                gint           height);
-void          gdk_window_clear_area_e          (GdkWindow     *window,
-                                                gint           x,
-                                                gint           y,
-                                                gint           width,
-                                                gint           height);
 void          gdk_window_raise                 (GdkWindow     *window);
 void          gdk_window_lower                 (GdkWindow     *window);
 void          gdk_window_restack               (GdkWindow     *window,
@@ -577,10 +592,6 @@ gboolean      gdk_window_ensure_native        (GdkWindow       *window);
  * This allows for making shaped (partially transparent) windows
  * - cool feature, needed for Drag and Drag for example.
  */
-void gdk_window_shape_combine_mask  (GdkWindow	      *window,
-                                     GdkBitmap	      *mask,
-                                     gint	       x,
-                                     gint	       y);
 void gdk_window_shape_combine_region (GdkWindow	      *window,
                                       const cairo_region_t *shape_region,
                                       gint	       offset_x,
@@ -608,10 +619,6 @@ void gdk_window_set_composited   (GdkWindow *window,
  */
 void gdk_window_merge_child_shapes         (GdkWindow       *window);
 
-void gdk_window_input_shape_combine_mask   (GdkWindow       *window,
-					    GdkBitmap       *mask,
-					    gint             x,
-					    gint             y);
 void gdk_window_input_shape_combine_region (GdkWindow       *window,
                                             const cairo_region_t *shape_region,
                                             gint             offset_x,
@@ -690,9 +697,8 @@ void          gdk_window_set_transient_for (GdkWindow     *window,
 					    GdkWindow     *parent);
 void	      gdk_window_set_background	 (GdkWindow	  *window,
 					  const GdkColor  *color);
-void	      gdk_window_set_back_pixmap (GdkWindow	  *window,
-					  GdkPixmap	  *pixmap,
-					  gboolean	   parent_relative);
+void	      gdk_window_set_background_pattern (GdkWindow	 *window,
+                                                 cairo_pattern_t *pattern);
 cairo_pattern_t *gdk_window_get_background_pattern (GdkWindow     *window);
 
 void	      gdk_window_set_cursor	 (GdkWindow	  *window,
@@ -711,6 +717,8 @@ void	      gdk_window_get_geometry	 (GdkWindow	  *window,
 					  gint		  *width,
 					  gint		  *height,
 					  gint		  *depth);
+int           gdk_window_get_width       (GdkWindow       *window);
+int           gdk_window_get_height      (GdkWindow       *window);
 void	      gdk_window_get_position	 (GdkWindow	  *window,
 					  gint		  *x,
 					  gint		  *y);
@@ -733,13 +741,6 @@ void       gdk_window_coords_from_parent (GdkWindow       *window,
                                           gdouble         *x,
                                           gdouble         *y);
 
-#if !defined (GDK_DISABLE_DEPRECATED) || defined (GTK_COMPILATION) || defined (GDK_COMPILATION)
-/* Used by gtk_handle_box_button_changed () */
-gboolean      gdk_window_get_deskrelative_origin (GdkWindow	  *window,
-					  gint		  *x,
-					  gint		  *y);
-#endif
-
 void	      gdk_window_get_root_origin (GdkWindow	  *window,
 					  gint		  *x,
 					  gint		  *y);
@@ -776,10 +777,6 @@ GdkEventMask  gdk_window_get_device_events (GdkWindow    *window,
 
 void          gdk_window_set_icon_list   (GdkWindow       *window,
 					  GList           *pixbufs);
-void	      gdk_window_set_icon	 (GdkWindow	  *window, 
-					  GdkWindow	  *icon_window,
-					  GdkPixmap	  *pixmap,
-					  GdkBitmap	  *mask);
 void	      gdk_window_set_icon_name	 (GdkWindow	  *window, 
 					  const gchar	  *name);
 void	      gdk_window_set_group	 (GdkWindow	  *window, 
@@ -834,11 +831,26 @@ void       gdk_window_invalidate_rect           (GdkWindow          *window,
 void       gdk_window_invalidate_region         (GdkWindow          *window,
 					         const cairo_region_t    *region,
 					         gboolean            invalidate_children);
-void       gdk_window_invalidate_maybe_recurse  (GdkWindow          *window,
-						 const cairo_region_t    *region,
-						 gboolean (*child_func) (GdkWindow *, gpointer),
-						 gpointer   user_data);
-cairo_region_t *gdk_window_get_update_area     (GdkWindow    *window);
+
+/**
+ * GdkWindowChildFunc:
+ * @window: a #GdkWindow
+ * @user_data: user data
+ *
+ * A function of this type is passed to gdk_window_invalidate_maybe_recurse().
+ * It gets called for each child of the window to determine whether to
+ * recursively invalidate it or now.
+ *
+ * Returns: %TRUE to invalidate @window recursively
+ */
+typedef gboolean (*GdkWindowChildFunc)          (GdkWindow *window,
+                                                 gpointer   user_data);
+
+void       gdk_window_invalidate_maybe_recurse  (GdkWindow            *window,
+						 const cairo_region_t *region,
+						 GdkWindowChildFunc    child_func,
+						 gpointer              user_data);
+cairo_region_t *gdk_window_get_update_area      (GdkWindow            *window);
 
 void       gdk_window_freeze_updates      (GdkWindow    *window);
 void       gdk_window_thaw_updates        (GdkWindow    *window);
@@ -860,11 +872,6 @@ void       gdk_window_constrain_size      (GdkGeometry  *geometry,
                                            gint         *new_width,
                                            gint         *new_height);
 
-void gdk_window_get_internal_paint_info (GdkWindow    *window,
-					 GdkDrawable **real_drawable,
-					 gint         *x_offset,
-					 gint         *y_offset);
-
 void gdk_window_enable_synchronized_configure (GdkWindow *window);
 void gdk_window_configure_finished            (GdkWindow *window);
 
@@ -875,22 +882,13 @@ GdkPointerHooks *gdk_set_pointer_hooks (const GdkPointerHooks *new_hooks);
 GdkWindow *gdk_get_default_root_window (void);
 
 /* Offscreen redirection */
-GdkPixmap *gdk_offscreen_window_get_pixmap     (GdkWindow     *window);
+cairo_surface_t *
+           gdk_offscreen_window_get_surface    (GdkWindow     *window);
 void       gdk_offscreen_window_set_embedder   (GdkWindow     *window,
 						GdkWindow     *embedder);
 GdkWindow *gdk_offscreen_window_get_embedder   (GdkWindow     *window);
 void       gdk_window_geometry_changed         (GdkWindow     *window);
 
-void       gdk_window_redirect_to_drawable   (GdkWindow     *window,
-                                              GdkDrawable   *drawable,
-                                              gint           src_x,
-                                              gint           src_y,
-                                              gint           dest_x,
-                                              gint           dest_y,
-                                              gint           width,
-                                              gint           height);
-void       gdk_window_remove_redirection     (GdkWindow     *window);
-
 /* Multidevice support */
 void       gdk_window_set_support_multidevice (GdkWindow *window,
                                                gboolean   support_multidevice);
