@@ -25,10 +25,11 @@
  */
 
 #include "config.h"
+
 #include "gdkx.h"
 #include "gdkprivate-x11.h"
 #include "gdkdisplay-x11.h"
-#include "gdkalias.h"
+
 #include <stdio.h>
 
 static guint     gdk_xid_hash  (XID *xid);
@@ -36,11 +37,6 @@ static gboolean  gdk_xid_equal (XID *a,
 				XID *b);
 
 
-/* The 3 high bits of XIDs are unused. We use one to mark fonts, 
- * since we must be able to skip fonts when iterating over all XIDs.
- */
-#define XID_FONT_BIT (1<<31)
-
 void
 _gdk_xid_table_insert (GdkDisplay *display,
 		       XID	  *xid,
@@ -77,14 +73,15 @@ _gdk_xid_table_remove (GdkDisplay *display,
     g_hash_table_remove (display_x11->xid_ht, &xid);
 }
 
-/** 
+/**
  * gdk_xid_table_lookup_for_display:
  * @display: the #GdkDisplay.
  * @xid: an X id.
  *
  * Returns the GDK object associated with the given X id.
  *
- * Returns: a GDK object associated with the given X id.
+ * Return value: the associated #GdkWindow or %NULL if no
+ *     object is associated with the X id.
  *
  * Since: 2.2
  */
@@ -109,12 +106,13 @@ gdk_xid_table_lookup_for_display (GdkDisplay  *display,
 /**
  * gdk_xid_table_lookup:
  * @xid: an X id.
- * 
- * Returns the Gdk object associated with the given X id.
- * 
- * Return value: the associated Gdk object, which may be a #GdkPixmap,
- * a #GdkWindow or a #GdkFont.
- **/
+ *
+ * Returns the Gdk object associated with the given X id for the default
+ * display.
+ *
+ * Return value: the associated #GdkWindow or %NULL if no
+ *     object is associated with the X id.
+ */
 gpointer
 gdk_xid_table_lookup (XID xid)
 {
@@ -131,8 +129,5 @@ static gboolean
 gdk_xid_equal (XID *a,
 	       XID *b)
 {
-  return ((*a & ~XID_FONT_BIT) == (*b & ~XID_FONT_BIT));
+  return (*a == *b);
 }
-
-#define __GDK_XID_C__
-#include "gdkaliasdef.c"
