@@ -50,8 +50,8 @@ static GtkCellEditable * gtk_cell_renderer_spin_start_editing (GtkCellRenderer
 							       GdkEvent            *event,
 							       GtkWidget           *widget,
 							       const gchar         *path,
-							       GdkRectangle        *background_area,
-							       GdkRectangle        *cell_area,
+							       const GdkRectangle  *background_area,
+							       const GdkRectangle  *cell_area,
 							       GtkCellRendererState flags);
 enum {
   PROP_0,
@@ -259,12 +259,12 @@ gtk_cell_renderer_spin_key_press_event (GtkWidget   *widget,
 {
   if (event->state == 0)
     {
-      if (event->keyval == GDK_Up)
+      if (event->keyval == GDK_KEY_Up)
 	{
 	  gtk_spin_button_spin (GTK_SPIN_BUTTON (widget), GTK_SPIN_STEP_FORWARD, 1);
 	  return TRUE;
 	}
-      else if (event->keyval == GDK_Down)
+      else if (event->keyval == GDK_KEY_Down)
 	{
 	  gtk_spin_button_spin (GTK_SPIN_BUTTON (widget), GTK_SPIN_STEP_BACKWARD, 1);
 	  return TRUE;
@@ -290,13 +290,13 @@ gtk_cell_renderer_spin_button_press_event (GtkWidget      *widget,
 }
 
 static GtkCellEditable *
-gtk_cell_renderer_spin_start_editing (GtkCellRenderer     *cell,
-				      GdkEvent            *event,
-				      GtkWidget           *widget,
-				      const gchar         *path,
-				      GdkRectangle        *background_area,
-				      GdkRectangle        *cell_area,
-				      GtkCellRendererState flags)
+gtk_cell_renderer_spin_start_editing (GtkCellRenderer      *cell,
+				      GdkEvent             *event,
+				      GtkWidget            *widget,
+				      const gchar          *path,
+				      const GdkRectangle   *background_area,
+				      const GdkRectangle   *cell_area,
+				      GtkCellRendererState  flags)
 {
   GtkCellRendererSpinPrivate *priv;
   GtkCellRendererText *cell_text;
