@@ -24,62 +24,34 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
-#include "gtkalias.h"
+#include "config.h"
+
 #include "gtkcheckbutton.h"
-#include "gtkintl.h"
+
 #include "gtklabel.h"
 
+#include "gtkprivate.h"
+#include "gtkintl.h"
+
 
 #define INDICATOR_SIZE     13
 #define INDICATOR_SPACING  2
 
 
-static void gtk_check_button_class_init          (GtkCheckButtonClass *klass);
-static void gtk_check_button_init                (GtkCheckButton      *check_button);
 static void gtk_check_button_size_request        (GtkWidget           *widget,
 						  GtkRequisition      *requisition);
 static void gtk_check_button_size_allocate       (GtkWidget           *widget,
 						  GtkAllocation       *allocation);
-static gint gtk_check_button_expose              (GtkWidget           *widget,
-						  GdkEventExpose      *event);
+static gboolean gtk_check_button_draw            (GtkWidget           *widget,
+						  cairo_t             *cr);
 static void gtk_check_button_paint               (GtkWidget           *widget,
-						  GdkRectangle        *area);
+						  cairo_t             *cr);
 static void gtk_check_button_draw_indicator      (GtkCheckButton      *check_button,
-						  GdkRectangle        *area);
+						  cairo_t             *cr);
 static void gtk_real_check_button_draw_indicator (GtkCheckButton      *check_button,
-						  GdkRectangle        *area);
+						  cairo_t             *cr);
 
-static GtkToggleButtonClass *parent_class = NULL;
-
-
-GType
-gtk_check_button_get_type (void)
-{
-  static GType check_button_type = 0;
-  
-  if (!check_button_type)
-    {
-      static const GTypeInfo check_button_info =
-      {
-	sizeof (GtkCheckButtonClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_check_button_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkCheckButton),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_check_button_init,
-      };
-      
-      check_button_type =
-	g_type_register_static (GTK_TYPE_TOGGLE_BUTTON, "GtkCheckButton",
-				&check_button_info, 0);
-    }
-  
-  return check_button_type;
-}
+G_DEFINE_TYPE (GtkCheckButton, gtk_check_button, GTK_TYPE_TOGGLE_BUTTON)
 
 static void
 gtk_check_button_class_init (GtkCheckButtonClass *class)
@@ -87,37 +59,36 @@ gtk_check_button_class_init (GtkCheckButtonClass *class)
   GtkWidgetClass *widget_class;
   
   widget_class = (GtkWidgetClass*) class;
-  parent_class = g_type_class_peek_parent (class);
   
   widget_class->size_request = gtk_check_button_size_request;
   widget_class->size_allocate = gtk_check_button_size_allocate;
-  widget_class->expose_event = gtk_check_button_expose;
+  widget_class->draw = gtk_check_button_draw;
 
   class->draw_indicator = gtk_real_check_button_draw_indicator;
 
   gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_int ("indicator_size",
+					   g_param_spec_int ("indicator-size",
 							     P_("Indicator Size"),
 							     P_("Size of check or radio indicator"),
 							     0,
 							     G_MAXINT,
 							     INDICATOR_SIZE,
-							     G_PARAM_READABLE));
+							     GTK_PARAM_READABLE));
   gtk_widget_class_install_style_property (widget_class,
-					   g_param_spec_int ("indicator_spacing",
+					   g_param_spec_int ("indicator-spacing",
 							     P_("Indicator Spacing"),
 							     P_("Spacing around check or radio indicator"),
 							     0,
 							     G_MAXINT,
 							     INDICATOR_SPACING,
-							     G_PARAM_READABLE));
+							     GTK_PARAM_READABLE));
 }
 
 static void
 gtk_check_button_init (GtkCheckButton *check_button)
 {
-  GTK_WIDGET_SET_FLAGS (check_button, GTK_NO_WINDOW);
-  GTK_WIDGET_UNSET_FLAGS (check_button, GTK_RECEIVES_DEFAULT);
+  gtk_widget_set_has_window (GTK_WIDGET (check_button), FALSE);
+  gtk_widget_set_receives_default (GTK_WIDGET (check_button), FALSE);
   GTK_TOGGLE_BUTTON (check_button)->draw_indicator = TRUE;
   GTK_BUTTON (check_button)->depress_on_activate = FALSE;
 }
@@ -148,7 +119,10 @@ gtk_check_button_new_with_label (const gchar *label)
 GtkWidget*
 gtk_check_button_new_with_mnemonic (const gchar *label)
 {
-  return g_object_new (GTK_TYPE_CHECK_BUTTON, "label", label, "use_underline", TRUE, NULL);
+  return g_object_new (GTK_TYPE_CHECK_BUTTON, 
+                       "label", label, 
+                       "use-underline", TRUE, 
+                       NULL);
 }
 
 
@@ -157,45 +131,53 @@ gtk_check_button_new_with_mnemonic (const gchar *label)
  */
 static void
 gtk_check_button_paint (GtkWidget    *widget,
-			GdkRectangle *area)
+			cairo_t      *cr)
 {
   GtkCheckButton *check_button = GTK_CHECK_BUTTON (widget);
-  
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      gint border_width;
-      gint interior_focus;
-      gint focus_width;
-      gint focus_pad;
-	  
-      gtk_widget_style_get (widget,
-			    "interior-focus", &interior_focus,
-			    "focus-line-width", &focus_width,
-			    "focus-padding", &focus_pad,
-			    NULL);
-
-      gtk_check_button_draw_indicator (check_button, area);
+  gint border_width;
+  gint interior_focus;
+  gint focus_width;
+  gint focus_pad;
       
-      border_width = GTK_CONTAINER (widget)->border_width;
-      if (GTK_WIDGET_HAS_FOCUS (widget))
-	{
-	  GtkWidget *child = GTK_BIN (widget)->child;
-	  
-	  if (interior_focus && child && GTK_WIDGET_VISIBLE (child))
-	    gtk_paint_focus (widget->style, widget->window, GTK_WIDGET_STATE (widget),
-			     NULL, widget, "checkbutton",
-			     child->allocation.x - focus_width - focus_pad,
-			     child->allocation.y - focus_width - focus_pad,
-			     child->allocation.width + 2 * (focus_width + focus_pad),
-			     child->allocation.height + 2 * (focus_width + focus_pad));
-	  else
-	    gtk_paint_focus (widget->style, widget->window, GTK_WIDGET_STATE (widget), 
-			     NULL, widget, "checkbutton",
-			     border_width + widget->allocation.x,
-			     border_width + widget->allocation.y,
-			     widget->allocation.width - 2 * border_width,
-			     widget->allocation.height - 2 * border_width);
-	}
+  gtk_widget_style_get (widget,
+                        "interior-focus", &interior_focus,
+                        "focus-line-width", &focus_width,
+                        "focus-padding", &focus_pad,
+                        NULL);
+
+  gtk_check_button_draw_indicator (check_button, cr);
+
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+  if (gtk_widget_has_focus (widget))
+    {
+      GtkStateType state = gtk_widget_get_state (widget);
+      GtkStyle *style = gtk_widget_get_style (widget);
+      GtkWidget *child = gtk_bin_get_child (GTK_BIN (widget));
+      GtkAllocation allocation;
+
+      gtk_widget_get_allocation (widget, &allocation);
+
+      if (interior_focus && child && gtk_widget_get_visible (child))
+        {
+          GtkAllocation child_allocation;
+
+          gtk_widget_get_allocation (child, &child_allocation);
+          gtk_paint_focus (style, cr, state,
+                           widget, "checkbutton",
+                           child_allocation.x - allocation.x - focus_width - focus_pad,
+                           child_allocation.y - allocation.y - focus_width - focus_pad,
+                           child_allocation.width + 2 * (focus_width + focus_pad),
+                           child_allocation.height + 2 * (focus_width + focus_pad));
+        }
+      else
+        {
+          gtk_paint_focus (style, cr, state,
+                           widget, "checkbutton",
+                           border_width,
+                           border_width,
+                           allocation.width - 2 * border_width,
+                           allocation.height - 2 * border_width);
+        }
     }
 }
 
@@ -207,10 +189,10 @@ _gtk_check_button_get_props (GtkCheckButton *check_button,
   GtkWidget *widget =  GTK_WIDGET (check_button);
 
   if (indicator_size)
-    gtk_widget_style_get (widget, "indicator_size", indicator_size, NULL);
+    gtk_widget_style_get (widget, "indicator-size", indicator_size, NULL);
 
   if (indicator_spacing)
-    gtk_widget_style_get (widget, "indicator_spacing", indicator_spacing, NULL);
+    gtk_widget_style_get (widget, "indicator-spacing", indicator_spacing, NULL);
 }
 
 static void
@@ -225,9 +207,11 @@ gtk_check_button_size_request (GtkWidget      *widget,
       gint temp;
       gint indicator_size;
       gint indicator_spacing;
-      gint border_width = GTK_CONTAINER (widget)->border_width;
       gint focus_width;
       gint focus_pad;
+      guint border_width;
+
+      border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
       gtk_widget_style_get (GTK_WIDGET (widget),
 			    "focus-line-width", &focus_width,
@@ -240,12 +224,12 @@ gtk_check_button_size_request (GtkWidget      *widget,
       _gtk_check_button_get_props (GTK_CHECK_BUTTON (widget),
  				   &indicator_size, &indicator_spacing);
       
-      child = GTK_BIN (widget)->child;
-      if (child && GTK_WIDGET_VISIBLE (child))
+      child = gtk_bin_get_child (GTK_BIN (widget));
+      if (child && gtk_widget_get_visible (child))
 	{
 	  GtkRequisition child_requisition;
-	  
-	  gtk_widget_size_request (child, &child_requisition);
+
+          gtk_widget_get_preferred_size (child, &child_requisition, NULL);
 
 	  requisition->width += child_requisition.width + indicator_spacing;
 	  requisition->height += child_requisition.height;
@@ -257,7 +241,7 @@ gtk_check_button_size_request (GtkWidget      *widget,
       requisition->height = MAX (requisition->height, temp) + 2 * (focus_width + focus_pad);
     }
   else
-    (* GTK_WIDGET_CLASS (parent_class)->size_request) (widget, requisition);
+    GTK_WIDGET_CLASS (gtk_check_button_parent_class)->size_request (widget, requisition);
 }
 
 static void
@@ -275,6 +259,7 @@ gtk_check_button_size_allocate (GtkWidget     *widget,
 
   if (toggle_button->draw_indicator)
     {
+      GtkWidget *child;
       gint indicator_size;
       gint indicator_spacing;
       gint focus_width;
@@ -285,20 +270,22 @@ gtk_check_button_size_allocate (GtkWidget     *widget,
 			    "focus-line-width", &focus_width,
 			    "focus-padding", &focus_pad,
 			    NULL);
-						    
-      widget->allocation = *allocation;
-      if (GTK_WIDGET_REALIZED (widget))
+
+      gtk_widget_set_allocation (widget, allocation);
+
+      if (gtk_widget_get_realized (widget))
 	gdk_window_move_resize (button->event_window,
 				allocation->x, allocation->y,
 				allocation->width, allocation->height);
-      
-      if (GTK_BIN (button)->child && GTK_WIDGET_VISIBLE (GTK_BIN (button)->child))
+
+      child = gtk_bin_get_child (GTK_BIN (button));
+      if (child && gtk_widget_get_visible (child))
 	{
 	  GtkRequisition child_requisition;
- 	  gint border_width = GTK_CONTAINER (widget)->border_width;
+          guint border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+
+          gtk_widget_get_preferred_size (child, &child_requisition, NULL);
 
-	  gtk_widget_get_child_requisition (GTK_BIN (button)->child, &child_requisition);
- 
 	  child_allocation.width = MIN (child_requisition.width,
 					allocation->width -
 					((border_width + focus_width + focus_pad) * 2
@@ -310,69 +297,61 @@ gtk_check_button_size_allocate (GtkWidget     *widget,
 	  child_allocation.height = MAX (child_allocation.height, 1);
 	  
 	  child_allocation.x = (border_width + indicator_size + indicator_spacing * 3 +
-				widget->allocation.x + focus_width + focus_pad);
-	  child_allocation.y = widget->allocation.y +
-		  (allocation->height - child_allocation.height) / 2;
+                                allocation->x + focus_width + focus_pad);
+          child_allocation.y = allocation->y + (allocation->height - child_allocation.height) / 2;
 
 	  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
 	    child_allocation.x = allocation->x + allocation->width
 	      - (child_allocation.x - allocation->x + child_allocation.width);
 	  
-	  gtk_widget_size_allocate (GTK_BIN (button)->child, &child_allocation);
+	  gtk_widget_size_allocate (child, &child_allocation);
 	}
     }
   else
-    (* GTK_WIDGET_CLASS (parent_class)->size_allocate) (widget, allocation);
+    GTK_WIDGET_CLASS (gtk_check_button_parent_class)->size_allocate (widget, allocation);
 }
 
 static gint
-gtk_check_button_expose (GtkWidget      *widget,
-			 GdkEventExpose *event)
+gtk_check_button_draw (GtkWidget *widget,
+                       cairo_t   *cr)
 {
-  GtkCheckButton *check_button;
   GtkToggleButton *toggle_button;
   GtkBin *bin;
+  GtkWidget *child;
   
-  check_button = GTK_CHECK_BUTTON (widget);
   toggle_button = GTK_TOGGLE_BUTTON (widget);
   bin = GTK_BIN (widget);
   
-  if (GTK_WIDGET_DRAWABLE (widget))
+  if (toggle_button->draw_indicator)
     {
-      if (toggle_button->draw_indicator)
-	{
-	  gtk_check_button_paint (widget, &event->area);
-	  
-	  if (bin->child)
-	    gtk_container_propagate_expose (GTK_CONTAINER (widget),
-					    bin->child,
-					    event);
-	}
-      else if (GTK_WIDGET_CLASS (parent_class)->expose_event)
-	(* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
+      gtk_check_button_paint (widget, cr);
+
+      child = gtk_bin_get_child (bin);
+      if (child)
+        gtk_container_propagate_draw (GTK_CONTAINER (widget),
+                                      child,
+                                      cr);
     }
-  
+  else if (GTK_WIDGET_CLASS (gtk_check_button_parent_class)->draw)
+    GTK_WIDGET_CLASS (gtk_check_button_parent_class)->draw (widget, cr);
+
   return FALSE;
 }
 
 
 static void
 gtk_check_button_draw_indicator (GtkCheckButton *check_button,
-				 GdkRectangle   *area)
+				 cairo_t        *cr)
 {
-  GtkCheckButtonClass *class;
-  
-  g_return_if_fail (GTK_IS_CHECK_BUTTON (check_button));
-  
-  class = GTK_CHECK_BUTTON_GET_CLASS (check_button);
-  
+  GtkCheckButtonClass *class = GTK_CHECK_BUTTON_GET_CLASS (check_button);
+
   if (class->draw_indicator)
-    (* class->draw_indicator) (check_button, area);
+    class->draw_indicator (check_button, cr);
 }
 
 static void
 gtk_real_check_button_draw_indicator (GtkCheckButton *check_button,
-				      GdkRectangle   *area)
+				      cairo_t        *cr)
 {
   GtkWidget *widget;
   GtkWidget *child;
@@ -385,69 +364,69 @@ gtk_real_check_button_draw_indicator (GtkCheckButton *check_button,
   gint indicator_spacing;
   gint focus_width;
   gint focus_pad;
+  guint border_width;
   gboolean interior_focus;
+  GtkAllocation allocation;
+  GtkStyle *style;
+  GdkWindow *window;
+
+  widget = GTK_WIDGET (check_button);
+  button = GTK_BUTTON (check_button);
+  toggle_button = GTK_TOGGLE_BUTTON (check_button);
+
+  gtk_widget_get_allocation (widget, &allocation);
+  style = gtk_widget_get_style (widget);
+  window = gtk_widget_get_window (widget);
+
+  gtk_widget_style_get (widget, 
+                        "interior-focus", &interior_focus,
+                        "focus-line-width", &focus_width, 
+                        "focus-padding", &focus_pad, 
+                        NULL);
+
+  _gtk_check_button_get_props (check_button, &indicator_size, &indicator_spacing);
+
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+
+  x = indicator_spacing + border_width;
+  y = (allocation.height - indicator_size) / 2;
+
+  child = gtk_bin_get_child (GTK_BIN (check_button));
+  if (!interior_focus || !(child && gtk_widget_get_visible (child)))
+    x += focus_width + focus_pad;      
+
+  if (toggle_button->inconsistent)
+    shadow_type = GTK_SHADOW_ETCHED_IN;
+  else if (toggle_button->active)
+    shadow_type = GTK_SHADOW_IN;
+  else
+    shadow_type = GTK_SHADOW_OUT;
+
+  if (button->activate_timeout || (button->button_down && button->in_button))
+    state_type = GTK_STATE_ACTIVE;
+  else if (button->in_button)
+    state_type = GTK_STATE_PRELIGHT;
+  else if (!gtk_widget_is_sensitive (widget))
+    state_type = GTK_STATE_INSENSITIVE;
+  else
+    state_type = GTK_STATE_NORMAL;
   
-  if (GTK_WIDGET_DRAWABLE (check_button))
+  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
+    x = allocation.width - (indicator_size + x);
+
+  if (gtk_widget_get_state (widget) == GTK_STATE_PRELIGHT)
     {
-      widget = GTK_WIDGET (check_button);
-      button = GTK_BUTTON (check_button);
-      toggle_button = GTK_TOGGLE_BUTTON (check_button);
-  
-      gtk_widget_style_get (widget, "interior_focus", &interior_focus,
-			    "focus-line-width", &focus_width, 
-			    "focus-padding", &focus_pad, NULL);
 
-      _gtk_check_button_get_props (check_button, &indicator_size, &indicator_spacing);
-
-      x = widget->allocation.x + indicator_spacing + GTK_CONTAINER (widget)->border_width;
-      y = widget->allocation.y + (widget->allocation.height - indicator_size) / 2;
-
-      child = GTK_BIN (check_button)->child;
-      if (!interior_focus || !(child && GTK_WIDGET_VISIBLE (child)))
-	x += focus_width + focus_pad;      
-
-      if (toggle_button->inconsistent)
-	shadow_type = GTK_SHADOW_ETCHED_IN;
-      else if (toggle_button->active)
-	shadow_type = GTK_SHADOW_IN;
-      else
-	shadow_type = GTK_SHADOW_OUT;
-
-      if (button->activate_timeout || (button->button_down && button->in_button))
-	state_type = GTK_STATE_ACTIVE;
-      else if (button->in_button)
-	state_type = GTK_STATE_PRELIGHT;
-      else if (!GTK_WIDGET_IS_SENSITIVE (widget))
-	state_type = GTK_STATE_INSENSITIVE;
-      else
-	state_type = GTK_STATE_NORMAL;
-      
-      if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)
-	x = widget->allocation.x + widget->allocation.width - (indicator_size + x - widget->allocation.x);
-
-      if (GTK_WIDGET_STATE (toggle_button) == GTK_STATE_PRELIGHT)
-	{
-	  GdkRectangle restrict_area;
-	  GdkRectangle new_area;
-	      
-	  restrict_area.x = widget->allocation.x + GTK_CONTAINER (widget)->border_width;
-	  restrict_area.y = widget->allocation.y + GTK_CONTAINER (widget)->border_width;
-	  restrict_area.width = widget->allocation.width - (2 * GTK_CONTAINER (widget)->border_width);
-	  restrict_area.height = widget->allocation.height - (2 * GTK_CONTAINER (widget)->border_width);
-	  
-	  if (gdk_rectangle_intersect (area, &restrict_area, &new_area))
-	    {
-	      gtk_paint_flat_box (widget->style, widget->window, GTK_STATE_PRELIGHT,
-				  GTK_SHADOW_ETCHED_OUT, 
-				  area, widget, "checkbutton",
-				  new_area.x, new_area.y,
-				  new_area.width, new_area.height);
-	    }
-	}
-
-      gtk_paint_check (widget->style, widget->window,
-		       state_type, shadow_type,
-		       area, widget, "checkbutton",
-		       x, y, indicator_size, indicator_size);
+      gtk_paint_flat_box (style, cr, GTK_STATE_PRELIGHT,
+                          GTK_SHADOW_ETCHED_OUT, 
+                          widget, "checkbutton",
+                          border_width, border_width,
+                          allocation.width - (2 * border_width),
+                          allocation.height - (2 * border_width));
     }
+
+  gtk_paint_check (style, cr,
+                   state_type, shadow_type,
+                   widget, "checkbutton",
+                   x, y, indicator_size, indicator_size);
 }
