@@ -35,7 +35,6 @@
 #include <gdk/gdkkeysyms.h>
 #include "gtkdnd.h"
 
-#define GTK_EXPANDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GTK_TYPE_EXPANDER, GtkExpanderPrivate))
 
 #define DEFAULT_EXPANDER_SIZE 10
 #define DEFAULT_EXPANDER_SPACING 2
@@ -48,7 +47,8 @@ enum
   PROP_USE_UNDERLINE,
   PROP_USE_MARKUP,
   PROP_SPACING,
-  PROP_LABEL_WIDGET
+  PROP_LABEL_WIDGET,
+  PROP_LABEL_FILL
 };
 
 struct _GtkExpanderPrivate
@@ -66,6 +66,7 @@ struct _GtkExpanderPrivate
   guint             use_markup : 1; 
   guint             button_down : 1;
   guint             prelight : 1;
+  guint             label_fill : 1;
 };
 
 static void gtk_expander_set_property (GObject          *object,
@@ -77,18 +78,15 @@ static void gtk_expander_get_property (GObject          *object,
 				       GValue           *value,
 				       GParamSpec       *pspec);
 
-static void gtk_expander_destroy (GtkObject *object);
-
+static void     gtk_expander_destroy        (GtkWidget        *widget);
 static void     gtk_expander_realize        (GtkWidget        *widget);
 static void     gtk_expander_unrealize      (GtkWidget        *widget);
-static void     gtk_expander_size_request   (GtkWidget        *widget,
-					     GtkRequisition   *requisition);
 static void     gtk_expander_size_allocate  (GtkWidget        *widget,
 					     GtkAllocation    *allocation);
 static void     gtk_expander_map            (GtkWidget        *widget);
 static void     gtk_expander_unmap          (GtkWidget        *widget);
-static gboolean gtk_expander_expose         (GtkWidget        *widget,
-					     GdkEventExpose   *event);
+static gboolean gtk_expander_draw           (GtkWidget        *widget,
+					     cairo_t          *cr);
 static gboolean gtk_expander_button_press   (GtkWidget        *widget,
 					     GdkEventButton   *event);
 static gboolean gtk_expander_button_release (GtkWidget        *widget,
@@ -133,6 +131,23 @@ static void gtk_expander_buildable_add_child      (GtkBuildable *buildable,
 						   GObject      *child,
 						   const gchar  *type);
 
+
+/* GtkWidget      */
+static void  gtk_expander_get_preferred_width             (GtkWidget           *widget,
+                                                           gint                *minimum_size,
+                                                           gint                *natural_size);
+static void  gtk_expander_get_preferred_height            (GtkWidget           *widget,
+                                                           gint                *minimum_size,
+                                                           gint                *natural_size);
+static void  gtk_expander_get_preferred_height_for_width  (GtkWidget           *layout,
+                                                           gint                 width,
+                                                           gint                *minimum_height,
+                                                           gint                *natural_height);
+static void  gtk_expander_get_preferred_width_for_height  (GtkWidget           *layout,
+                                                           gint                 width,
+                                                           gint                *minimum_height,
+                                                           gint                *natural_height);
+
 G_DEFINE_TYPE_WITH_CODE (GtkExpander, gtk_expander, GTK_TYPE_BIN,
 			 G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
 						gtk_expander_buildable_init))
@@ -141,27 +156,23 @@ static void
 gtk_expander_class_init (GtkExpanderClass *klass)
 {
   GObjectClass *gobject_class;
-  GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
   GtkContainerClass *container_class;
 
   gobject_class   = (GObjectClass *) klass;
-  object_class    = (GtkObjectClass *) klass;
   widget_class    = (GtkWidgetClass *) klass;
   container_class = (GtkContainerClass *) klass;
 
   gobject_class->set_property = gtk_expander_set_property;
   gobject_class->get_property = gtk_expander_get_property;
 
-  object_class->destroy = gtk_expander_destroy;
-
+  widget_class->destroy              = gtk_expander_destroy;
   widget_class->realize              = gtk_expander_realize;
   widget_class->unrealize            = gtk_expander_unrealize;
-  widget_class->size_request         = gtk_expander_size_request;
   widget_class->size_allocate        = gtk_expander_size_allocate;
   widget_class->map                  = gtk_expander_map;
   widget_class->unmap                = gtk_expander_unmap;
-  widget_class->expose_event         = gtk_expander_expose;
+  widget_class->draw                 = gtk_expander_draw;
   widget_class->button_press_event   = gtk_expander_button_press;
   widget_class->button_release_event = gtk_expander_button_release;
   widget_class->enter_notify_event   = gtk_expander_enter_notify;
@@ -171,6 +182,10 @@ gtk_expander_class_init (GtkExpanderClass *klass)
   widget_class->state_changed        = gtk_expander_state_changed;
   widget_class->drag_motion          = gtk_expander_drag_motion;
   widget_class->drag_leave           = gtk_expander_drag_leave;
+  widget_class->get_preferred_width            = gtk_expander_get_preferred_width;
+  widget_class->get_preferred_height           = gtk_expander_get_preferred_height;
+  widget_class->get_preferred_height_for_width = gtk_expander_get_preferred_height_for_width;
+  widget_class->get_preferred_width_for_height = gtk_expander_get_preferred_width_for_height;
 
   container_class->add    = gtk_expander_add;
   container_class->remove = gtk_expander_remove;
@@ -230,6 +245,14 @@ gtk_expander_class_init (GtkExpanderClass *klass)
 							GTK_TYPE_WIDGET,
 							GTK_PARAM_READWRITE));
 
+  g_object_class_install_property (gobject_class,
+				   PROP_LABEL_FILL,
+				   g_param_spec_boolean ("label-fill",
+							 P_("Label fill"),
+							 P_("Whether the label widget should fill all available horizontal space"),
+							 FALSE,
+							 GTK_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
   gtk_widget_class_install_style_property (widget_class,
 					   g_param_spec_int ("expander-size",
 							     P_("Expander Size"),
@@ -263,7 +286,9 @@ gtk_expander_init (GtkExpander *expander)
 {
   GtkExpanderPrivate *priv;
 
-  expander->priv = priv = GTK_EXPANDER_GET_PRIVATE (expander);
+  expander->priv = priv = G_TYPE_INSTANCE_GET_PRIVATE (expander,
+                                                       GTK_TYPE_EXPANDER,
+                                                       GtkExpanderPrivate);
 
   gtk_widget_set_can_focus (GTK_WIDGET (expander), TRUE);
   gtk_widget_set_has_window (GTK_WIDGET (expander), FALSE);
@@ -280,6 +305,7 @@ gtk_expander_init (GtkExpander *expander)
   priv->use_markup = FALSE;
   priv->button_down = FALSE;
   priv->prelight = FALSE;
+  priv->label_fill = FALSE;
   priv->expand_timer = 0;
 
   gtk_drag_dest_set (GTK_WIDGET (expander), 0, NULL, 0, 0);
@@ -334,6 +360,9 @@ gtk_expander_set_property (GObject      *object,
     case PROP_LABEL_WIDGET:
       gtk_expander_set_label_widget (expander, g_value_get_object (value));
       break;
+    case PROP_LABEL_FILL:
+      gtk_expander_set_label_fill (expander, g_value_get_boolean (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -371,6 +400,9 @@ gtk_expander_get_property (GObject    *object,
 			  priv->label_widget ?
 			  G_OBJECT (priv->label_widget) : NULL);
       break;
+    case PROP_LABEL_FILL:
+      g_value_set_boolean (value, priv->label_fill);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -378,23 +410,25 @@ gtk_expander_get_property (GObject    *object,
 }
 
 static void
-gtk_expander_destroy (GtkObject *object)
+gtk_expander_destroy (GtkWidget *widget)
 {
-  GtkExpanderPrivate *priv = GTK_EXPANDER (object)->priv;
-  
+  GtkExpanderPrivate *priv = GTK_EXPANDER (widget)->priv;
+
   if (priv->animation_timeout)
     {
       g_source_remove (priv->animation_timeout);
       priv->animation_timeout = 0;
     }
-  
-  GTK_OBJECT_CLASS (gtk_expander_parent_class)->destroy (object);
+
+  GTK_WIDGET_CLASS (gtk_expander_parent_class)->destroy (widget);
 }
 
 static void
 gtk_expander_realize (GtkWidget *widget)
 {
+  GtkAllocation allocation;
   GtkExpanderPrivate *priv;
+  GdkWindow *window;
   GdkWindowAttr attributes;
   gint attributes_mask;
   gint border_width;
@@ -402,9 +436,10 @@ gtk_expander_realize (GtkWidget *widget)
   gint label_height;
 
   priv = GTK_EXPANDER (widget)->priv;
+
   gtk_widget_set_realized (widget, TRUE);
 
-  border_width = GTK_CONTAINER (widget)->border_width;
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
   get_expander_bounds (GTK_EXPANDER (widget), &expander_rect);
   
@@ -412,16 +447,19 @@ gtk_expander_realize (GtkWidget *widget)
     {
       GtkRequisition label_requisition;
 
-      gtk_widget_get_child_requisition (priv->label_widget, &label_requisition);
+      gtk_widget_get_preferred_size (priv->label_widget,
+                                     &label_requisition, NULL);
       label_height = label_requisition.height;
     }
   else
     label_height = 0;
 
+  gtk_widget_get_allocation (widget, &allocation);
+
   attributes.window_type = GDK_WINDOW_CHILD;
-  attributes.x = widget->allocation.x + border_width;
-  attributes.y = widget->allocation.y + border_width;
-  attributes.width = MAX (widget->allocation.width - 2 * border_width, 1);
+  attributes.x = allocation.x + border_width;
+  attributes.y = allocation.y + border_width;
+  attributes.width = MAX (allocation.width - 2 * border_width, 1);
   attributes.height = MAX (expander_rect.height, label_height - 2 * border_width);
   attributes.wclass = GDK_INPUT_ONLY;
   attributes.event_mask = gtk_widget_get_events (widget)     |
@@ -432,14 +470,15 @@ gtk_expander_realize (GtkWidget *widget)
 
   attributes_mask = GDK_WA_X | GDK_WA_Y;
 
-  widget->window = gtk_widget_get_parent_window (widget);
-  g_object_ref (widget->window);
+  window = gtk_widget_get_parent_window (widget);
+  gtk_widget_set_window (widget, window);
+  g_object_ref (window);
 
   priv->event_window = gdk_window_new (gtk_widget_get_parent_window (widget),
 				       &attributes, attributes_mask);
   gdk_window_set_user_data (priv->event_window, widget);
 
-  widget->style = gtk_style_attach (widget->style, widget->window);
+  gtk_widget_style_attach (widget);
 }
 
 static void
@@ -457,71 +496,11 @@ gtk_expander_unrealize (GtkWidget *widget)
   GTK_WIDGET_CLASS (gtk_expander_parent_class)->unrealize (widget);
 }
 
-static void
-gtk_expander_size_request (GtkWidget      *widget,
-			   GtkRequisition *requisition)
-{
-  GtkExpander *expander;
-  GtkBin *bin;
-  GtkExpanderPrivate *priv;
-  gint border_width;
-  gint expander_size;
-  gint expander_spacing;
-  gboolean interior_focus;
-  gint focus_width;
-  gint focus_pad;
-
-  bin = GTK_BIN (widget);
-  expander = GTK_EXPANDER (widget);
-  priv = expander->priv;
-
-  border_width = GTK_CONTAINER (widget)->border_width;
-
-  gtk_widget_style_get (widget,
-			"interior-focus", &interior_focus,
-			"focus-line-width", &focus_width,
-			"focus-padding", &focus_pad,
-			"expander-size", &expander_size,
-			"expander-spacing", &expander_spacing,
-			NULL);
-
-  requisition->width = expander_size + 2 * expander_spacing +
-		       2 * focus_width + 2 * focus_pad;
-  requisition->height = interior_focus ? (2 * focus_width + 2 * focus_pad) : 0;
-
-  if (priv->label_widget && gtk_widget_get_visible (priv->label_widget))
-    {
-      GtkRequisition label_requisition;
-
-      gtk_widget_size_request (priv->label_widget, &label_requisition);
-
-      requisition->width  += label_requisition.width;
-      requisition->height += label_requisition.height;
-    }
-
-  requisition->height = MAX (expander_size + 2 * expander_spacing, requisition->height);
-
-  if (!interior_focus)
-    requisition->height += 2 * focus_width + 2 * focus_pad;
-
-  if (bin->child && GTK_WIDGET_CHILD_VISIBLE (bin->child))
-    {
-      GtkRequisition child_requisition;
-
-      gtk_widget_size_request (bin->child, &child_requisition);
-
-      requisition->width = MAX (requisition->width, child_requisition.width);
-      requisition->height += child_requisition.height + priv->spacing;
-    }
-
-  requisition->width  += 2 * border_width;
-  requisition->height += 2 * border_width;
-}
-
 static void
 get_expander_bounds (GtkExpander  *expander,
 		     GdkRectangle *rect)
 {
+  GtkAllocation allocation;
   GtkWidget *widget;
   GtkExpanderPrivate *priv;
   gint border_width;
@@ -535,7 +514,9 @@ get_expander_bounds (GtkExpander  *expander,
   widget = GTK_WIDGET (expander);
   priv = expander->priv;
 
-  border_width = GTK_CONTAINER (expander)->border_width;
+  gtk_widget_get_allocation (widget, &allocation);
+
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
 
   gtk_widget_style_get (widget,
 			"interior-focus", &interior_focus,
@@ -547,20 +528,20 @@ get_expander_bounds (GtkExpander  *expander,
 
   ltr = gtk_widget_get_direction (widget) != GTK_TEXT_DIR_RTL;
 
-  rect->x = widget->allocation.x + border_width;
-  rect->y = widget->allocation.y + border_width;
+  rect->x = allocation.x + border_width;
+  rect->y = allocation.y + border_width;
 
   if (ltr)
     rect->x += expander_spacing;
   else
-    rect->x += widget->allocation.width - 2 * border_width -
+    rect->x += allocation.width - 2 * border_width -
                expander_spacing - expander_size;
 
   if (priv->label_widget && gtk_widget_get_visible (priv->label_widget))
     {
       GtkAllocation label_allocation;
 
-      label_allocation = priv->label_widget->allocation;
+      gtk_widget_get_allocation (priv->label_widget, &label_allocation);
 
       if (expander_size < label_allocation.height)
 	rect->y += focus_width + focus_pad + (label_allocation.height - expander_size) / 2;
@@ -589,23 +570,26 @@ gtk_expander_size_allocate (GtkWidget     *widget,
 			    GtkAllocation *allocation)
 {
   GtkExpander *expander;
-  GtkBin *bin;
+  GtkWidget *child;
   GtkExpanderPrivate *priv;
-  GtkRequisition child_requisition;
   gboolean child_visible = FALSE;
-  gint border_width;
+  guint border_width;
   gint expander_size;
   gint expander_spacing;
   gboolean interior_focus;
   gint focus_width;
   gint focus_pad;
-  gint label_height;
+  gint label_height, top_min_height;
+  gint label_xpad, label_xoffset;
+  gint child_ypad, child_yoffset;
 
   expander = GTK_EXPANDER (widget);
-  bin = GTK_BIN (widget);
-  priv = expander->priv;
+  child    = gtk_bin_get_child (GTK_BIN (widget));
+  priv     = expander->priv;
 
-  border_width = GTK_CONTAINER (widget)->border_width;
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+
+  gtk_widget_set_allocation (widget, allocation);
 
   gtk_widget_style_get (widget,
 			"interior-focus", &interior_focus,
@@ -615,45 +599,48 @@ gtk_expander_size_allocate (GtkWidget     *widget,
 			"expander-spacing", &expander_spacing,
 			NULL);
 
-  child_requisition.width = 0;
-  child_requisition.height = 0;
-  if (bin->child && GTK_WIDGET_CHILD_VISIBLE (bin->child))
-    {
-      child_visible = TRUE;
-      gtk_widget_get_child_requisition (bin->child, &child_requisition);
-    }
 
-  widget->allocation = *allocation;
+  /* Calculate some offsets/padding first */
+  label_xoffset = border_width + expander_size + focus_width + 2 * expander_spacing + focus_pad;
+  label_xpad = 2 * border_width + expander_size + 2 * focus_width + 2 * expander_spacing + 2 * focus_pad;
+
+  child_yoffset  = border_width + priv->spacing + (interior_focus ? 0 : 2 * focus_width + 2 * focus_pad);
+  child_ypad     = 2 * border_width + priv->spacing + (interior_focus ? 0 : 2 * focus_width + 2 * focus_pad);
+  top_min_height = 2 * expander_spacing + expander_size;
+
+  child_visible = (child && gtk_widget_get_child_visible (child));
 
   if (priv->label_widget && gtk_widget_get_visible (priv->label_widget))
     {
       GtkAllocation label_allocation;
-      GtkRequisition label_requisition;
+      gint          natural_label_width;
       gboolean ltr;
 
-      gtk_widget_get_child_requisition (priv->label_widget, &label_requisition);
+      gtk_widget_get_preferred_width (priv->label_widget, NULL, &natural_label_width);
+
+      if (priv->label_fill)
+        label_allocation.width = allocation->width - label_xpad;
+      else
+        label_allocation.width = MIN (natural_label_width, allocation->width - label_xpad);
+      label_allocation.width = MAX (label_allocation.width, 1);
+
+      /* We distribute the minimum height to the label widget and prioritize
+       * the child widget giving it the remaining height */
+      gtk_widget_get_preferred_height_for_width (priv->label_widget,
+                                                 label_allocation.width, &label_height, NULL);
 
       ltr = gtk_widget_get_direction (widget) != GTK_TEXT_DIR_RTL;
 
-      if (ltr)
-	label_allocation.x = (widget->allocation.x +
-                              border_width + focus_width + focus_pad +
-                              expander_size + 2 * expander_spacing);
+      if (priv->label_fill)
+        label_allocation.x = allocation->x + label_xoffset;
+      else if (ltr)
+        label_allocation.x = allocation->x + label_xoffset;
       else
-        label_allocation.x = (widget->allocation.x + widget->allocation.width -
-                              (label_requisition.width +
-                               border_width + focus_width + focus_pad +
-                               expander_size + 2 * expander_spacing));
+        label_allocation.x = allocation->x + allocation->width -
+                             (label_allocation.width + label_xoffset);
 
-      label_allocation.y = widget->allocation.y + border_width + focus_width + focus_pad;
-
-      label_allocation.width = MIN (label_requisition.width,
-				    allocation->width - 2 * border_width -
-				    expander_size - 2 * expander_spacing -
-				    2 * focus_width - 2 * focus_pad);
-      label_allocation.width = MAX (label_allocation.width, 1);
-
-      label_allocation.height = MIN (label_requisition.height,
+      label_allocation.y = allocation->y + border_width + focus_width + focus_pad;
+      label_allocation.height = MIN (label_height,
 				     allocation->height - 2 * border_width -
 				     2 * focus_width - 2 * focus_pad -
 				     (child_visible ? priv->spacing : 0));
@@ -675,9 +662,9 @@ gtk_expander_size_allocate (GtkWidget     *widget,
       get_expander_bounds (expander, &rect);
 
       gdk_window_move_resize (priv->event_window,
-			      allocation->x + border_width, 
-			      allocation->y + border_width, 
-			      MAX (allocation->width - 2 * border_width, 1), 
+			      allocation->x + border_width,
+			      allocation->y + border_width,
+			      MAX (allocation->width - 2 * border_width, 1),
 			      MAX (rect.height, label_height - 2 * border_width));
     }
 
@@ -686,24 +673,17 @@ gtk_expander_size_allocate (GtkWidget     *widget,
       GtkAllocation child_allocation;
       gint top_height;
 
-      top_height = MAX (2 * expander_spacing + expander_size,
-			label_height +
-			(interior_focus ? 2 * focus_width + 2 * focus_pad : 0));
+      top_height = MAX (top_min_height,
+			label_height + (interior_focus ? 2 * focus_width + 2 * focus_pad : 0));
 
-      child_allocation.x = widget->allocation.x + border_width;
-      child_allocation.y = widget->allocation.y + border_width + top_height + priv->spacing;
-
-      if (!interior_focus)
-	child_allocation.y += 2 * focus_width + 2 * focus_pad;
+      child_allocation.x = allocation->x + border_width;
+      child_allocation.y = allocation->y + top_height + child_yoffset;
 
       child_allocation.width = MAX (allocation->width - 2 * border_width, 1);
-
-      child_allocation.height = allocation->height - top_height -
-				2 * border_width - priv->spacing -
-				(!interior_focus ? 2 * focus_width + 2 * focus_pad : 0);
+      child_allocation.height = allocation->height - top_height - child_ypad;
       child_allocation.height = MAX (child_allocation.height, 1);
 
-      gtk_widget_size_allocate (bin->child, &child_allocation);
+      gtk_widget_size_allocate (child, &child_allocation);
     }
 }
 
@@ -736,8 +716,9 @@ gtk_expander_unmap (GtkWidget *widget)
 }
 
 static void
-gtk_expander_paint_prelight (GtkExpander *expander)
+gtk_expander_paint_prelight (GtkExpander *expander, cairo_t *cr)
 {
+  GtkAllocation allocation;
   GtkWidget *widget;
   GtkContainer *container;
   GtkExpanderPrivate *priv;
@@ -747,6 +728,7 @@ gtk_expander_paint_prelight (GtkExpander *expander)
   int focus_pad;
   int expander_size;
   int expander_spacing;
+  guint border_width;
 
   priv = expander->priv;
   widget = GTK_WIDGET (expander);
@@ -760,12 +742,20 @@ gtk_expander_paint_prelight (GtkExpander *expander)
 			"expander-spacing", &expander_spacing,
 			NULL);
 
-  area.x = widget->allocation.x + container->border_width;
-  area.y = widget->allocation.y + container->border_width;
-  area.width = widget->allocation.width - (2 * container->border_width);
+  gtk_widget_get_allocation (widget, &allocation);
+
+  border_width = gtk_container_get_border_width (container);
+  area.x = border_width;
+  area.y = border_width;
+  area.width = allocation.width - (2 * border_width);
 
   if (priv->label_widget && gtk_widget_get_visible (priv->label_widget))
-    area.height = priv->label_widget->allocation.height;
+    {
+      GtkAllocation label_widget_allocation;
+
+      gtk_widget_get_allocation (priv->label_widget, &label_widget_allocation);
+      area.height = label_widget_allocation.height;
+    }
   else
     area.height = 0;
 
@@ -773,47 +763,49 @@ gtk_expander_paint_prelight (GtkExpander *expander)
   area.height = MAX (area.height, expander_size + 2 * expander_spacing);
   area.height += !interior_focus ? (focus_width + focus_pad) * 2 : 0;
 
-  gtk_paint_flat_box (widget->style, widget->window,
+  gtk_paint_flat_box (gtk_widget_get_style (widget),
+                      cr,
 		      GTK_STATE_PRELIGHT,
 		      GTK_SHADOW_ETCHED_OUT,
-		      &area, widget, "expander",
+		      widget, "expander",
 		      area.x, area.y,
 		      area.width, area.height);
 }
 
 static void
-gtk_expander_paint (GtkExpander *expander)
+gtk_expander_paint (GtkExpander *expander, cairo_t *cr)
 {
   GtkWidget *widget;
   GdkRectangle clip;
+  GtkAllocation allocation;
   GtkStateType state;
 
   widget = GTK_WIDGET (expander);
 
   get_expander_bounds (expander, &clip);
+  gtk_widget_get_allocation (widget, &allocation);
 
-  state = widget->state;
+  state = gtk_widget_get_state (widget);
   if (expander->priv->prelight)
     {
       state = GTK_STATE_PRELIGHT;
 
-      gtk_expander_paint_prelight (expander);
+      gtk_expander_paint_prelight (expander, cr);
     }
 
-  gtk_paint_expander (widget->style,
-		      widget->window,
+  gtk_paint_expander (gtk_widget_get_style (widget),
+		      cr,
 		      state,
-		      &clip,
 		      widget,
 		      "expander",
-		      clip.x + clip.width / 2,
-		      clip.y + clip.height / 2,
+		      clip.x + clip.width / 2 - allocation.x,
+		      clip.y + clip.height / 2 - allocation.y,
 		      expander->priv->expander_style);
 }
 
 static void
-gtk_expander_paint_focus (GtkExpander  *expander,
-			  GdkRectangle *area)
+gtk_expander_paint_focus (GtkExpander *expander,
+			  cairo_t     *cr)
 {
   GtkWidget *widget;
   GtkExpanderPrivate *priv;
@@ -826,11 +818,13 @@ gtk_expander_paint_focus (GtkExpander  *expander,
   gint expander_size;
   gint expander_spacing;
   gboolean ltr;
+  GtkAllocation allocation;
 
   widget = GTK_WIDGET (expander);
   priv = expander->priv;
 
-  border_width = GTK_CONTAINER (widget)->border_width;
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+  gtk_widget_get_allocation (widget, &allocation);
 
   gtk_widget_style_get (widget,
 			"interior-focus", &interior_focus,
@@ -848,8 +842,9 @@ gtk_expander_paint_focus (GtkExpander  *expander,
     {
       if (gtk_widget_get_visible (priv->label_widget))
 	{
-	  GtkAllocation label_allocation = priv->label_widget->allocation;
+	  GtkAllocation label_allocation;
 
+	  gtk_widget_get_allocation (priv->label_widget, &label_allocation);
 	  width  = label_allocation.width;
 	  height = label_allocation.height;
 	}
@@ -857,8 +852,8 @@ gtk_expander_paint_focus (GtkExpander  *expander,
       width  += 2 * focus_pad + 2 * focus_width;
       height += 2 * focus_pad + 2 * focus_width;
 
-      x = widget->allocation.x + border_width;
-      y = widget->allocation.y + border_width;
+      x = border_width;
+      y = border_width;
 
       if (ltr)
 	{
@@ -867,7 +862,7 @@ gtk_expander_paint_focus (GtkExpander  *expander,
 	}
       else
 	{
-	  x += widget->allocation.width - 2 * border_width
+	  x += allocation.width - 2 * border_width
 	    - expander_spacing * 2 - expander_size - width;
 	}
 
@@ -881,32 +876,31 @@ gtk_expander_paint_focus (GtkExpander  *expander,
     {
       get_expander_bounds (expander, &rect);
 
-      x = rect.x - focus_pad;
-      y = rect.y - focus_pad;
+      x = rect.x - allocation.x - focus_pad;
+      y = rect.y - allocation.y - focus_pad;
       width = rect.width + 2 * focus_pad;
       height = rect.height + 2 * focus_pad;
     }
-      
-  gtk_paint_focus (widget->style, widget->window, gtk_widget_get_state (widget),
-		   area, widget, "expander",
+
+  gtk_paint_focus (gtk_widget_get_style (widget),
+                   cr,
+                   gtk_widget_get_state (widget),
+		   widget, "expander",
 		   x, y, width, height);
 }
 
 static gboolean
-gtk_expander_expose (GtkWidget      *widget,
-		     GdkEventExpose *event)
+gtk_expander_draw (GtkWidget *widget,
+		   cairo_t   *cr)
 {
-  if (gtk_widget_is_drawable (widget))
-    {
-      GtkExpander *expander = GTK_EXPANDER (widget);
+  GtkExpander *expander = GTK_EXPANDER (widget);
 
-      gtk_expander_paint (expander);
+  gtk_expander_paint (expander, cr);
 
-      if (gtk_widget_has_focus (widget))
-	gtk_expander_paint_focus (expander, &event->area);
+  if (gtk_widget_has_focus (widget))
+    gtk_expander_paint_focus (expander, cr);
 
-      GTK_WIDGET_CLASS (gtk_expander_parent_class)->expose_event (widget, event);
-    }
+  GTK_WIDGET_CLASS (gtk_expander_parent_class)->draw (widget, cr);
 
   return FALSE;
 }
@@ -961,12 +955,14 @@ gtk_expander_state_changed (GtkWidget    *widget,
 static void
 gtk_expander_redraw_expander (GtkExpander *expander)
 {
-  GtkWidget *widget;
-
-  widget = GTK_WIDGET (expander);
+  GtkAllocation allocation;
+  GtkWidget *widget = GTK_WIDGET (expander);
 
   if (gtk_widget_get_realized (widget))
-    gdk_window_invalidate_rect (widget->window, &widget->allocation, FALSE);
+    {
+      gtk_widget_get_allocation (widget, &allocation);
+      gdk_window_invalidate_rect (gtk_widget_get_window (widget), &allocation, FALSE);
+    }
 }
 
 static gboolean
@@ -1080,7 +1076,7 @@ focus_current_site (GtkExpander      *expander,
 {
   GtkWidget *current_focus;
 
-  current_focus = GTK_CONTAINER (expander)->focus_child;
+  current_focus = gtk_container_get_focus_child (GTK_CONTAINER (expander));
 
   if (!current_focus)
     return FALSE;
@@ -1107,7 +1103,7 @@ focus_in_site (GtkExpander      *expander,
       {
 	GtkWidget *child = gtk_bin_get_child (GTK_BIN (expander));
 
-	if (child && GTK_WIDGET_CHILD_VISIBLE (child))
+	if (child && gtk_widget_get_child_visible (child))
 	  return gtk_widget_child_focus (child, direction);
 	else
 	  return FALSE;
@@ -1204,7 +1200,7 @@ gtk_expander_focus (GtkWidget        *widget,
       FocusSite site = FOCUS_NONE;
       
       widget_is_focus = gtk_widget_is_focus (widget);
-      old_focus_child = GTK_CONTAINER (widget)->focus_child;
+      old_focus_child = gtk_container_get_focus_child (GTK_CONTAINER (widget));
       
       if (old_focus_child && old_focus_child == expander->priv->label_widget)
 	site = FOCUS_LABEL;
@@ -1255,9 +1251,11 @@ gtk_expander_forall (GtkContainer *container,
 {
   GtkBin *bin = GTK_BIN (container);
   GtkExpanderPrivate *priv = GTK_EXPANDER (container)->priv;
+  GtkWidget *child;
 
-  if (bin->child)
-    (* callback) (bin->child, callback_data);
+  child = gtk_bin_get_child (bin);
+  if (child)
+    (* callback) (child, callback_data);
 
   if (priv->label_widget)
     (* callback) (priv->label_widget, callback_data);
@@ -1269,6 +1267,223 @@ gtk_expander_activate (GtkExpander *expander)
   gtk_expander_set_expanded (expander, !expander->priv->expanded);
 }
 
+
+static void     
+gtk_expander_get_preferred_width (GtkWidget *widget,
+                                  gint      *minimum_size,
+                                  gint      *natural_size)
+{
+  GtkExpander *expander;
+  GtkWidget *child;
+  GtkExpanderPrivate *priv;
+  gint border_width;
+  gint expander_size;
+  gint expander_spacing;
+  gboolean interior_focus;
+  gint focus_width;
+  gint focus_pad;
+
+  child = gtk_bin_get_child (GTK_BIN (widget));
+  expander = GTK_EXPANDER (widget);
+  priv = expander->priv;
+
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+
+  gtk_widget_style_get (GTK_WIDGET (widget),
+			"interior-focus", &interior_focus,
+			"focus-line-width", &focus_width,
+			"focus-padding", &focus_pad,
+			"expander-size", &expander_size,
+			"expander-spacing", &expander_spacing,
+			NULL);
+
+  *minimum_size = *natural_size = 
+    expander_size + 2 * expander_spacing +
+    2 * focus_width + 2 * focus_pad;
+
+  if (priv->label_widget && gtk_widget_get_visible (priv->label_widget))
+    {
+      gint label_min, label_nat;
+
+      gtk_widget_get_preferred_width (priv->label_widget, 
+                                      &label_min, &label_nat);
+
+      *minimum_size += label_min;
+      *natural_size += label_nat;
+    }
+
+  if (child && gtk_widget_get_child_visible (child))
+    {
+      gint child_min, child_nat;
+
+      gtk_widget_get_preferred_width (child, 
+                                      &child_min, &child_nat);
+
+      *minimum_size = MAX (*minimum_size, child_min);
+      *natural_size = MAX (*natural_size, child_nat);
+
+    }
+
+  *minimum_size += 2 * border_width;
+  *natural_size += 2 * border_width;
+}
+
+static void
+gtk_expander_get_preferred_height (GtkWidget *widget,
+                                   gint      *minimum_size,
+                                   gint      *natural_size)
+{  
+  GtkExpander *expander;
+  GtkWidget *child;
+  GtkExpanderPrivate *priv;
+  gint border_width;
+  gint expander_size;
+  gint expander_spacing;
+  gboolean interior_focus;
+  gint focus_width;
+  gint focus_pad;
+
+  child = gtk_bin_get_child (GTK_BIN (widget));
+  expander = GTK_EXPANDER (widget);
+  priv = expander->priv;
+
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+
+  gtk_widget_style_get (GTK_WIDGET (widget),
+			"interior-focus", &interior_focus,
+			"focus-line-width", &focus_width,
+			"focus-padding", &focus_pad,
+			"expander-size", &expander_size,
+			"expander-spacing", &expander_spacing,
+			NULL);
+
+  *minimum_size = *natural_size = 
+    interior_focus ? (2 * focus_width + 2 * focus_pad) : 0;
+
+
+  if (priv->label_widget && gtk_widget_get_visible (priv->label_widget))
+    {
+      gint label_min, label_nat;
+
+      gtk_widget_get_preferred_height (priv->label_widget, 
+                                       &label_min, &label_nat);
+      
+      *minimum_size += label_min;
+      *natural_size += label_nat;
+    }
+
+  *minimum_size = MAX (*minimum_size, expander_size + 2 * expander_spacing);
+  *natural_size = MAX (*natural_size, *minimum_size);
+
+  if (!interior_focus)
+    {
+      gint extra = 2 * focus_width + 2 * focus_pad;
+      *minimum_size += extra;
+      *natural_size += extra;
+    }
+
+  if (child && gtk_widget_get_child_visible (child))
+    {
+      gint child_min, child_nat;
+
+      gtk_widget_get_preferred_height (child, 
+                                       &child_min, &child_nat);
+
+      *minimum_size += child_min + priv->spacing;
+      *natural_size += child_nat + priv->spacing;
+
+    }
+
+  *minimum_size += 2 * border_width;
+  *natural_size += 2 * border_width;
+}
+
+static void
+gtk_expander_get_preferred_height_for_width (GtkWidget *widget,
+                                             gint       width,
+                                             gint      *minimum_height,
+                                             gint      *natural_height)
+{
+  GtkExpander *expander;
+  GtkWidget *child;
+  GtkExpanderPrivate *priv;
+  gint border_width;
+  gint expander_size;
+  gint expander_spacing;
+  gboolean interior_focus;
+  gint focus_width;
+  gint focus_pad;
+  gint label_xpad;
+
+  child = gtk_bin_get_child (GTK_BIN (widget));
+  expander = GTK_EXPANDER (widget);
+  priv = expander->priv;
+
+  border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+
+  gtk_widget_style_get (GTK_WIDGET (widget),
+			"interior-focus", &interior_focus,
+			"focus-line-width", &focus_width,
+			"focus-padding", &focus_pad,
+			"expander-size", &expander_size,
+			"expander-spacing", &expander_spacing,
+			NULL);
+
+  label_xpad = 2 * border_width + expander_size + 2 * expander_spacing - 2 * focus_width + 2 * focus_pad;
+
+  *minimum_height = *natural_height = 
+    interior_focus ? (2 * focus_width + 2 * focus_pad) : 0;
+
+
+  if (priv->label_widget && gtk_widget_get_visible (priv->label_widget))
+    {
+      gint label_min, label_nat;
+
+      gtk_widget_get_preferred_height_for_width (priv->label_widget, 
+                                                 MAX (width - label_xpad, 1), 
+                                                 &label_min, &label_nat);
+      
+      *minimum_height += label_min;
+      *natural_height += label_nat;
+    }
+
+  *minimum_height = MAX (*minimum_height, expander_size + 2 * expander_spacing);
+  *natural_height = MAX (*natural_height, *minimum_height);
+
+  if (!interior_focus)
+    {
+      gint extra = 2 * focus_width + 2 * focus_pad;
+      *minimum_height += extra;
+      *natural_height += extra;
+    }
+
+  if (child && gtk_widget_get_child_visible (child))
+    {
+      gint child_min, child_nat;
+
+      gtk_widget_get_preferred_height_for_width (child, 
+                                                 MAX (width - 2 * border_width, 1), 
+                                                 &child_min, &child_nat);
+
+      *minimum_height += child_min + priv->spacing;
+      *natural_height += child_nat + priv->spacing;
+    }
+
+  *minimum_height += 2 * border_width;
+  *natural_height += 2 * border_width;
+}
+
+static void
+gtk_expander_get_preferred_width_for_height (GtkWidget *widget,
+                                             gint       height,
+                                             gint      *minimum_width,
+                                             gint      *natural_width)
+{
+  GTK_WIDGET_GET_CLASS (widget)->get_preferred_width (widget, minimum_width, natural_width);
+}
+
+
+
 /**
  * gtk_expander_new:
  * @label: the text of the label
@@ -1314,13 +1529,15 @@ static gboolean
 gtk_expander_animation_timeout (GtkExpander *expander)
 {
   GtkExpanderPrivate *priv = expander->priv;
+  GtkWidget *widget = GTK_WIDGET (expander);
+  GtkWidget *child;
   GdkRectangle area;
   gboolean finish = FALSE;
 
-  if (gtk_widget_get_realized (GTK_WIDGET (expander)))
+  if (gtk_widget_get_realized (widget))
     {
       get_expander_bounds (expander, &area);
-      gdk_window_invalidate_rect (GTK_WIDGET (expander)->window, &area, TRUE);
+      gdk_window_invalidate_rect (gtk_widget_get_window (widget), &area, TRUE);
     }
 
   if (priv->expanded)
@@ -1351,9 +1568,11 @@ gtk_expander_animation_timeout (GtkExpander *expander)
   if (finish)
     {
       priv->animation_timeout = 0;
-      if (GTK_BIN (expander)->child)
-	gtk_widget_set_child_visible (GTK_BIN (expander)->child, priv->expanded);
-      gtk_widget_queue_resize (GTK_WIDGET (expander));
+
+      child = gtk_bin_get_child (GTK_BIN (expander));
+      if (child)
+	gtk_widget_set_child_visible (child, priv->expanded);
+      gtk_widget_queue_resize (widget);
     }
 
   return !finish;
@@ -1389,6 +1608,7 @@ gtk_expander_set_expanded (GtkExpander *expander,
 			   gboolean     expanded)
 {
   GtkExpanderPrivate *priv;
+  GtkWidget *child;
 
   g_return_if_fail (GTK_IS_EXPANDER (expander));
 
@@ -1414,9 +1634,10 @@ gtk_expander_set_expanded (GtkExpander *expander,
 	  priv->expander_style = expanded ? GTK_EXPANDER_EXPANDED :
 					    GTK_EXPANDER_COLLAPSED;
 
-	  if (GTK_BIN (expander)->child)
+          child = gtk_bin_get_child (GTK_BIN (expander));
+	  if (child)
 	    {
-	      gtk_widget_set_child_visible (GTK_BIN (expander)->child, priv->expanded);
+	      gtk_widget_set_child_visible (child, priv->expanded);
 	      gtk_widget_queue_resize (GTK_WIDGET (expander));
 	    }
 	}
@@ -1689,7 +1910,7 @@ gtk_expander_set_label_widget (GtkExpander *expander,
 
   g_return_if_fail (GTK_IS_EXPANDER (expander));
   g_return_if_fail (label_widget == NULL || GTK_IS_WIDGET (label_widget));
-  g_return_if_fail (label_widget == NULL || label_widget->parent == NULL);
+  g_return_if_fail (label_widget == NULL || gtk_widget_get_parent (label_widget) == NULL);
 
   priv = expander->priv;
 
@@ -1731,8 +1952,9 @@ gtk_expander_set_label_widget (GtkExpander *expander,
  * Retrieves the label widget for the frame. See
  * gtk_expander_set_label_widget().
  *
- * Return value: the label widget, or %NULL if there is none.
- * 
+ * Return value: (transfer none): the label widget,
+ *     or %NULL if there is none.
+ *
  * Since: 2.4
  **/
 GtkWidget *
@@ -1742,3 +1964,58 @@ gtk_expander_get_label_widget (GtkExpander *expander)
 
   return expander->priv->label_widget;
 }
+
+/**
+ * gtk_expander_set_label_fill:
+ * @expander: a #GtkExpander
+ * @label_fill: %TRUE if the label should should fill all available horizontal
+ *              space
+ *
+ * Sets whether the label widget should fill all available horizontal space
+ * allocated to @expander.
+ *
+ * Since: 2.22
+ */
+void
+gtk_expander_set_label_fill (GtkExpander *expander,
+                             gboolean     label_fill)
+{
+  GtkExpanderPrivate *priv;
+
+  g_return_if_fail (GTK_IS_EXPANDER (expander));
+
+  priv = expander->priv;
+
+  label_fill = label_fill != FALSE;
+
+  if (priv->label_fill != label_fill)
+    {
+      priv->label_fill = label_fill;
+
+      if (priv->label_widget != NULL)
+        gtk_widget_queue_resize (GTK_WIDGET (expander));
+
+      g_object_notify (G_OBJECT (expander), "label-fill");
+    }
+}
+
+/**
+ * gtk_expander_get_label_fill:
+ * @expander: a #GtkExpander
+ *
+ * Returns whether the label widget will fill all available horizontal
+ * space allocated to @expander.
+ *
+ * Return value: %TRUE if the label widget will fill all available horizontal
+ *               space
+ *
+ * Since: 2.22
+ */
+gboolean
+gtk_expander_get_label_fill (GtkExpander *expander)
+{
+  g_return_val_if_fail (GTK_IS_EXPANDER (expander), FALSE);
+
+  return expander->priv->label_fill;
+}
+
