@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
@@ -26,7 +26,7 @@
 #define __GTK_FILE_CHOOSER_DIALOG_H__
 
 #include <gtk/gtkdialog.h>
-#include "gtkfilechooser.h"
+#include <gtk/gtkfilechooser.h>
 
 G_BEGIN_DECLS
 
@@ -59,12 +59,6 @@ GtkWidget *gtk_file_chooser_dialog_new              (const gchar          *title
 						     GtkFileChooserAction  action,
 						     const gchar          *first_button_text,
 						     ...) G_GNUC_NULL_TERMINATED;
-GtkWidget *gtk_file_chooser_dialog_new_with_backend (const gchar          *title,
-						     GtkWindow            *parent,
-						     GtkFileChooserAction  action,
-						     const gchar          *backend,
-						     const gchar          *first_button_text,
-						     ...) G_GNUC_NULL_TERMINATED;
 
 G_END_DECLS
 
