@@ -29,16 +29,39 @@
  */
 
 #include "config.h"
-#include "gdkconfig.h"
 
 #include "gtklayout.h"
+
+#include "gdkconfig.h"
+
 #include "gtkprivate.h"
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
-#include "gtkalias.h"
+
 
 typedef struct _GtkLayoutChild   GtkLayoutChild;
 
+struct _GtkLayoutPrivate
+{
+  /* Properties */
+  guint width;
+  guint height;
+
+  GtkAdjustment *hadjustment;
+  GtkAdjustment *vadjustment;
+  /* Properties */
+
+  GdkVisibilityState visibility;
+  GdkWindow *bin_window;
+
+  GList *children;
+
+  gint scroll_x;
+  gint scroll_y;
+
+  guint freeze_count;
+};
+
 struct _GtkLayoutChild {
   GtkWidget *widget;
   gint x;
@@ -78,8 +101,8 @@ static void gtk_layout_size_request       (GtkWidget      *widget,
                                            GtkRequisition *requisition);
 static void gtk_layout_size_allocate      (GtkWidget      *widget,
                                            GtkAllocation  *allocation);
-static gint gtk_layout_expose             (GtkWidget      *widget,
-                                           GdkEventExpose *event);
+static gint gtk_layout_draw               (GtkWidget      *widget,
+                                           cairo_t        *cr);
 static void gtk_layout_add                (GtkContainer   *container,
 					   GtkWidget      *widget);
 static void gtk_layout_remove             (GtkContainer   *container,
@@ -157,7 +180,7 @@ gtk_layout_get_bin_window (GtkLayout *layout)
 {
   g_return_val_if_fail (GTK_IS_LAYOUT (layout), NULL);
 
-  return layout->bin_window;
+  return layout->priv->bin_window;
 }
 
 /**
@@ -178,7 +201,7 @@ gtk_layout_get_hadjustment (GtkLayout *layout)
 {
   g_return_val_if_fail (GTK_IS_LAYOUT (layout), NULL);
 
-  return layout->hadjustment;
+  return layout->priv->hadjustment;
 }
 /**
  * gtk_layout_get_vadjustment:
@@ -198,13 +221,13 @@ gtk_layout_get_vadjustment (GtkLayout *layout)
 {
   g_return_val_if_fail (GTK_IS_LAYOUT (layout), NULL);
 
-  return layout->vadjustment;
+  return layout->priv->vadjustment;
 }
 
 static GtkAdjustment *
 new_default_adjustment (void)
 {
-  return GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 0.0, 0.0, 0.0, 0.0));
+  return gtk_adjustment_new (0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
 }
 
 static void           
@@ -212,54 +235,53 @@ gtk_layout_set_adjustments (GtkLayout     *layout,
 			    GtkAdjustment *hadj,
 			    GtkAdjustment *vadj)
 {
+  GtkLayoutPrivate *priv = layout->priv;
   gboolean need_adjust = FALSE;
 
-  g_return_if_fail (GTK_IS_LAYOUT (layout));
-
   if (hadj)
     g_return_if_fail (GTK_IS_ADJUSTMENT (hadj));
-  else if (layout->hadjustment)
+  else if (priv->hadjustment)
     hadj = new_default_adjustment ();
   if (vadj)
     g_return_if_fail (GTK_IS_ADJUSTMENT (vadj));
-  else if (layout->vadjustment)
+  else if (priv->vadjustment)
     vadj = new_default_adjustment ();
-  
-  if (layout->hadjustment && (layout->hadjustment != hadj))
+
+  if (priv->hadjustment && (priv->hadjustment != hadj))
     {
-      g_signal_handlers_disconnect_by_func (layout->hadjustment,
+      g_signal_handlers_disconnect_by_func (priv->hadjustment,
 					    gtk_layout_adjustment_changed,
 					    layout);
-      g_object_unref (layout->hadjustment);
+      g_object_unref (priv->hadjustment);
     }
-  
-  if (layout->vadjustment && (layout->vadjustment != vadj))
+
+  if (priv->vadjustment && (priv->vadjustment != vadj))
     {
-      g_signal_handlers_disconnect_by_func (layout->vadjustment,
+      g_signal_handlers_disconnect_by_func (priv->vadjustment,
 					    gtk_layout_adjustment_changed,
 					    layout);
-      g_object_unref (layout->vadjustment);
+      g_object_unref (priv->vadjustment);
     }
-  
-  if (layout->hadjustment != hadj)
+
+  if (priv->hadjustment != hadj)
     {
-      layout->hadjustment = hadj;
-      g_object_ref_sink (layout->hadjustment);
-      gtk_layout_set_adjustment_upper (layout->hadjustment, layout->width, FALSE);
-      
-      g_signal_connect (layout->hadjustment, "value-changed",
+      priv->hadjustment = hadj;
+      g_object_ref_sink (priv->hadjustment);
+      gtk_layout_set_adjustment_upper (priv->hadjustment, priv->width, FALSE);
+
+      g_signal_connect (priv->hadjustment, "value-changed",
 			G_CALLBACK (gtk_layout_adjustment_changed),
 			layout);
       need_adjust = TRUE;
     }
-  
-  if (layout->vadjustment != vadj)
+
+  if (priv->vadjustment != vadj)
     {
-      layout->vadjustment = vadj;
-      g_object_ref_sink (layout->vadjustment);
-      gtk_layout_set_adjustment_upper (layout->vadjustment, layout->height, FALSE);
-      
-      g_signal_connect (layout->vadjustment, "value-changed",
+      priv->vadjustment = vadj;
+      g_object_ref_sink (priv->vadjustment);
+      gtk_layout_set_adjustment_upper (priv->vadjustment, priv->height, FALSE);
+
+      g_signal_connect (priv->vadjustment, "value-changed",
 			G_CALLBACK (gtk_layout_adjustment_changed),
 			layout);
       need_adjust = TRUE;
@@ -275,9 +297,10 @@ static void
 gtk_layout_finalize (GObject *object)
 {
   GtkLayout *layout = GTK_LAYOUT (object);
+  GtkLayoutPrivate *priv = layout->priv;
 
-  g_object_unref (layout->hadjustment);
-  g_object_unref (layout->vadjustment);
+  g_object_unref (priv->hadjustment);
+  g_object_unref (priv->vadjustment);
 
   G_OBJECT_CLASS (gtk_layout_parent_class)->finalize (object);
 }
@@ -296,9 +319,13 @@ void
 gtk_layout_set_hadjustment (GtkLayout     *layout,
 			    GtkAdjustment *adjustment)
 {
+  GtkLayoutPrivate *priv;
+
   g_return_if_fail (GTK_IS_LAYOUT (layout));
 
-  gtk_layout_set_adjustments (layout, adjustment, layout->vadjustment);
+  priv = layout->priv;
+
+  gtk_layout_set_adjustments (layout, adjustment, priv->vadjustment);
   g_object_notify (G_OBJECT (layout), "hadjustment");
 }
  
@@ -316,9 +343,13 @@ void
 gtk_layout_set_vadjustment (GtkLayout     *layout,
 			    GtkAdjustment *adjustment)
 {
+  GtkLayoutPrivate *priv;
+
   g_return_if_fail (GTK_IS_LAYOUT (layout));
-  
-  gtk_layout_set_adjustments (layout, layout->hadjustment, adjustment);
+
+  priv = layout->priv;
+
+  gtk_layout_set_adjustments (layout, priv->hadjustment, adjustment);
   g_object_notify (G_OBJECT (layout), "vadjustment");
 }
 
@@ -326,9 +357,10 @@ static GtkLayoutChild*
 get_child (GtkLayout  *layout,
            GtkWidget  *widget)
 {
+  GtkLayoutPrivate *priv = layout->priv;
   GList *children;
-  
-  children = layout->children;
+
+  children = priv->children;
   while (children)
     {
       GtkLayoutChild *child;
@@ -360,21 +392,24 @@ gtk_layout_put (GtkLayout     *layout,
 		gint           x, 
 		gint           y)
 {
+  GtkLayoutPrivate *priv;
   GtkLayoutChild *child;
 
   g_return_if_fail (GTK_IS_LAYOUT (layout));
   g_return_if_fail (GTK_IS_WIDGET (child_widget));
-  
+
+  priv = layout->priv;
+
   child = g_new (GtkLayoutChild, 1);
 
   child->widget = child_widget;
   child->x = x;
   child->y = y;
 
-  layout->children = g_list_append (layout->children, child);
-  
+  priv->children = g_list_append (priv->children, child);
+
   if (gtk_widget_get_realized (GTK_WIDGET (layout)))
-    gtk_widget_set_parent_window (child->widget, layout->bin_window);
+    gtk_widget_set_parent_window (child->widget, priv->bin_window);
 
   gtk_widget_set_parent (child_widget, GTK_WIDGET (layout));
 }
@@ -432,7 +467,7 @@ gtk_layout_move (GtkLayout     *layout,
 {
   g_return_if_fail (GTK_IS_LAYOUT (layout));
   g_return_if_fail (GTK_IS_WIDGET (child_widget));
-  g_return_if_fail (child_widget->parent == GTK_WIDGET (layout));  
+  g_return_if_fail (gtk_widget_get_parent (child_widget) == GTK_WIDGET (layout));
 
   gtk_layout_move_internal (layout, child_widget, TRUE, x, TRUE, y);
 }
@@ -479,35 +514,40 @@ gtk_layout_set_size (GtkLayout     *layout,
 		     guint          width,
 		     guint          height)
 {
+  GtkLayoutPrivate *priv;
   GtkWidget *widget;
-  
+
   g_return_if_fail (GTK_IS_LAYOUT (layout));
-  
+
+  priv = layout->priv;
   widget = GTK_WIDGET (layout);
-  
+
   g_object_freeze_notify (G_OBJECT (layout));
-  if (width != layout->width)
+  if (width != priv->width)
      {
-	layout->width = width;
+	priv->width = width;
 	g_object_notify (G_OBJECT (layout), "width");
      }
-  if (height != layout->height)
+  if (height != priv->height)
      {
-	layout->height = height;
+	priv->height = height;
 	g_object_notify (G_OBJECT (layout), "height");
      }
   g_object_thaw_notify (G_OBJECT (layout));
 
-  if (layout->hadjustment)
-    gtk_layout_set_adjustment_upper (layout->hadjustment, layout->width, FALSE);
-  if (layout->vadjustment)
-    gtk_layout_set_adjustment_upper (layout->vadjustment, layout->height, FALSE);
+  if (priv->hadjustment)
+    gtk_layout_set_adjustment_upper (priv->hadjustment, priv->width, FALSE);
+  if (priv->vadjustment)
+    gtk_layout_set_adjustment_upper (priv->vadjustment, priv->height, FALSE);
 
   if (gtk_widget_get_realized (widget))
     {
-      width = MAX (width, widget->allocation.width);
-      height = MAX (height, widget->allocation.height);
-      gdk_window_resize (layout->bin_window, width, height);
+      GtkAllocation allocation;
+
+      gtk_widget_get_allocation (widget, &allocation);
+      width = MAX (width, allocation.width);
+      height = MAX (height, allocation.height);
+      gdk_window_resize (priv->bin_window, width, height);
     }
 }
 
@@ -526,48 +566,16 @@ gtk_layout_get_size (GtkLayout *layout,
 		     guint     *width,
 		     guint     *height)
 {
+  GtkLayoutPrivate *priv;
+
   g_return_if_fail (GTK_IS_LAYOUT (layout));
 
+  priv = layout->priv;
+
   if (width)
-    *width = layout->width;
+    *width = priv->width;
   if (height)
-    *height = layout->height;
-}
-
-/**
- * gtk_layout_freeze:
- * @layout: a #GtkLayout
- * 
- * This is a deprecated function, it doesn't do anything useful.
- **/
-void
-gtk_layout_freeze (GtkLayout *layout)
-{
-  g_return_if_fail (GTK_IS_LAYOUT (layout));
-
-  layout->freeze_count++;
-}
-
-/**
- * gtk_layout_thaw:
- * @layout: a #GtkLayout
- * 
- * This is a deprecated function, it doesn't do anything useful.
- **/
-void
-gtk_layout_thaw (GtkLayout *layout)
-{
-  g_return_if_fail (GTK_IS_LAYOUT (layout));
-
-  if (layout->freeze_count)
-    {
-      if (!(--layout->freeze_count))
-	{
-	  gtk_widget_queue_draw (GTK_WIDGET (layout));
-	  gdk_window_process_updates (GTK_WIDGET (layout)->window, TRUE);
-	}
-    }
-
+    *height = priv->height;
 }
 
 /* Basic Object handling procedures
@@ -650,7 +658,7 @@ gtk_layout_class_init (GtkLayoutClass *class)
   widget_class->map = gtk_layout_map;
   widget_class->size_request = gtk_layout_size_request;
   widget_class->size_allocate = gtk_layout_size_allocate;
-  widget_class->expose_event = gtk_layout_expose;
+  widget_class->draw = gtk_layout_draw;
   widget_class->style_set = gtk_layout_style_set;
 
   container_class->add = gtk_layout_add;
@@ -678,6 +686,8 @@ gtk_layout_class_init (GtkLayoutClass *class)
 		  G_TYPE_NONE, 2,
 		  GTK_TYPE_ADJUSTMENT,
 		  GTK_TYPE_ADJUSTMENT);
+
+  g_type_class_add_private (class, sizeof (GtkLayoutPrivate));
 }
 
 static void
@@ -687,20 +697,21 @@ gtk_layout_get_property (GObject     *object,
 			 GParamSpec  *pspec)
 {
   GtkLayout *layout = GTK_LAYOUT (object);
-  
+  GtkLayoutPrivate *priv = layout->priv;
+
   switch (prop_id)
     {
     case PROP_HADJUSTMENT:
-      g_value_set_object (value, layout->hadjustment);
+      g_value_set_object (value, priv->hadjustment);
       break;
     case PROP_VADJUSTMENT:
-      g_value_set_object (value, layout->vadjustment);
+      g_value_set_object (value, priv->vadjustment);
       break;
     case PROP_WIDTH:
-      g_value_set_uint (value, layout->width);
+      g_value_set_uint (value, priv->width);
       break;
     case PROP_HEIGHT:
-      g_value_set_uint (value, layout->height);
+      g_value_set_uint (value, priv->height);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -715,7 +726,8 @@ gtk_layout_set_property (GObject      *object,
 			 GParamSpec   *pspec)
 {
   GtkLayout *layout = GTK_LAYOUT (object);
-  
+  GtkLayoutPrivate *priv = layout->priv;
+
   switch (prop_id)
     {
     case PROP_HADJUSTMENT:
@@ -728,10 +740,10 @@ gtk_layout_set_property (GObject      *object,
       break;
     case PROP_WIDTH:
       gtk_layout_set_size (layout, g_value_get_uint (value),
-			   layout->height);
+			   priv->height);
       break;
     case PROP_HEIGHT:
-      gtk_layout_set_size (layout, layout->width,
+      gtk_layout_set_size (layout, priv->width,
 			   g_value_get_uint (value));
       break;
     default:
@@ -795,21 +807,28 @@ gtk_layout_get_child_property (GtkContainer *container,
 static void
 gtk_layout_init (GtkLayout *layout)
 {
-  layout->children = NULL;
+  GtkLayoutPrivate *priv;
 
-  layout->width = 100;
-  layout->height = 100;
+  layout->priv = G_TYPE_INSTANCE_GET_PRIVATE (layout,
+                                              GTK_TYPE_LAYOUT,
+                                              GtkLayoutPrivate);
+  priv = layout->priv;
 
-  layout->hadjustment = NULL;
-  layout->vadjustment = NULL;
+  priv->children = NULL;
 
-  layout->bin_window = NULL;
+  priv->width = 100;
+  priv->height = 100;
 
-  layout->scroll_x = 0;
-  layout->scroll_y = 0;
-  layout->visibility = GDK_VISIBILITY_PARTIAL;
+  priv->hadjustment = NULL;
+  priv->vadjustment = NULL;
 
-  layout->freeze_count = 0;
+  priv->bin_window = NULL;
+
+  priv->scroll_x = 0;
+  priv->scroll_y = 0;
+  priv->visibility = GDK_VISIBILITY_PARTIAL;
+
+  priv->freeze_count = 0;
 }
 
 static GObject *
@@ -817,6 +836,7 @@ gtk_layout_constructor (GType                  type,
 			guint                  n_properties,
 			GObjectConstructParam *properties)
 {
+  GtkLayoutPrivate *priv;
   GtkLayout *layout;
   GObject *object;
   GtkAdjustment *hadj, *vadj;
@@ -826,11 +846,12 @@ gtk_layout_constructor (GType                  type,
 								  properties);
 
   layout = GTK_LAYOUT (object);
+  priv = layout->priv;
 
-  hadj = layout->hadjustment ? layout->hadjustment : new_default_adjustment ();
-  vadj = layout->vadjustment ? layout->vadjustment : new_default_adjustment ();
+  hadj = priv->hadjustment ? priv->hadjustment : new_default_adjustment ();
+  vadj = priv->vadjustment ? priv->vadjustment : new_default_adjustment ();
 
-  if (!layout->hadjustment || !layout->vadjustment)
+  if (!priv->hadjustment || !priv->vadjustment)
     gtk_layout_set_adjustments (layout, hadj, vadj);
 
   return object;
@@ -843,50 +864,56 @@ static void
 gtk_layout_realize (GtkWidget *widget)
 {
   GtkLayout *layout = GTK_LAYOUT (widget);
-  GList *tmp_list;
+  GtkLayoutPrivate *priv = layout->priv;
+  GtkAllocation allocation;
+  GdkWindow *window;
   GdkWindowAttr attributes;
+  GList *tmp_list;
   gint attributes_mask;
 
   gtk_widget_set_realized (widget, TRUE);
 
+  gtk_widget_get_allocation (widget, &allocation);
+
   attributes.window_type = GDK_WINDOW_CHILD;
-  attributes.x = widget->allocation.x;
-  attributes.y = widget->allocation.y;
-  attributes.width = widget->allocation.width;
-  attributes.height = widget->allocation.height;
+  attributes.x = allocation.x;
+  attributes.y = allocation.y;
+  attributes.width = allocation.width;
+  attributes.height = allocation.height;
   attributes.wclass = GDK_INPUT_OUTPUT;
   attributes.visual = gtk_widget_get_visual (widget);
-  attributes.colormap = gtk_widget_get_colormap (widget);
   attributes.event_mask = GDK_VISIBILITY_NOTIFY_MASK;
 
-  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
+  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL;
 
-  widget->window = gdk_window_new (gtk_widget_get_parent_window (widget),
-				   &attributes, attributes_mask);
-  gdk_window_set_back_pixmap (widget->window, NULL, FALSE);
-  gdk_window_set_user_data (widget->window, widget);
+  window = gdk_window_new (gtk_widget_get_parent_window (widget),
+                           &attributes, attributes_mask);
+  gtk_widget_set_window (widget, window);
+  gdk_window_set_user_data (window, widget);
 
-  attributes.x = - layout->hadjustment->value,
-  attributes.y = - layout->vadjustment->value;
-  attributes.width = MAX (layout->width, widget->allocation.width);
-  attributes.height = MAX (layout->height, widget->allocation.height);
+  gtk_widget_get_allocation (widget, &allocation);
+
+  attributes.x = - priv->hadjustment->value,
+  attributes.y = - priv->vadjustment->value;
+  attributes.width = MAX (priv->width, allocation.width);
+  attributes.height = MAX (priv->height, allocation.height);
   attributes.event_mask = GDK_EXPOSURE_MASK | GDK_SCROLL_MASK | 
                           gtk_widget_get_events (widget);
 
-  layout->bin_window = gdk_window_new (widget->window,
-					&attributes, attributes_mask);
-  gdk_window_set_user_data (layout->bin_window, widget);
+  priv->bin_window = gdk_window_new (window,
+                                     &attributes, attributes_mask);
+  gdk_window_set_user_data (priv->bin_window, widget);
 
-  widget->style = gtk_style_attach (widget->style, widget->window);
-  gtk_style_set_background (widget->style, layout->bin_window, GTK_STATE_NORMAL);
+  gtk_widget_style_attach (widget);
+  gtk_style_set_background (gtk_widget_get_style (widget), priv->bin_window, GTK_STATE_NORMAL);
 
-  tmp_list = layout->children;
+  tmp_list = priv->children;
   while (tmp_list)
     {
       GtkLayoutChild *child = tmp_list->data;
       tmp_list = tmp_list->next;
 
-      gtk_widget_set_parent_window (child->widget, layout->bin_window);
+      gtk_widget_set_parent_window (child->widget, priv->bin_window);
     }
 }
 
@@ -894,11 +921,14 @@ static void
 gtk_layout_style_set (GtkWidget *widget,
                       GtkStyle  *old_style)
 {
+  GtkLayoutPrivate *priv;
+
   GTK_WIDGET_CLASS (gtk_layout_parent_class)->style_set (widget, old_style);
 
   if (gtk_widget_get_realized (widget))
     {
-      gtk_style_set_background (widget->style, GTK_LAYOUT (widget)->bin_window, GTK_STATE_NORMAL);
+      priv = GTK_LAYOUT (widget)->priv;
+      gtk_style_set_background (gtk_widget_get_style (widget), priv->bin_window, GTK_STATE_NORMAL);
     }
 }
 
@@ -906,11 +936,12 @@ static void
 gtk_layout_map (GtkWidget *widget)
 {
   GtkLayout *layout = GTK_LAYOUT (widget);
+  GtkLayoutPrivate *priv = layout->priv;
   GList *tmp_list;
 
   gtk_widget_set_mapped (widget, TRUE);
 
-  tmp_list = layout->children;
+  tmp_list = priv->children;
   while (tmp_list)
     {
       GtkLayoutChild *child = tmp_list->data;
@@ -923,18 +954,19 @@ gtk_layout_map (GtkWidget *widget)
 	}
     }
 
-  gdk_window_show (layout->bin_window);
-  gdk_window_show (widget->window);
+  gdk_window_show (priv->bin_window);
+  gdk_window_show (gtk_widget_get_window (widget));
 }
 
 static void 
 gtk_layout_unrealize (GtkWidget *widget)
 {
   GtkLayout *layout = GTK_LAYOUT (widget);
+  GtkLayoutPrivate *priv = layout->priv;
 
-  gdk_window_set_user_data (layout->bin_window, NULL);
-  gdk_window_destroy (layout->bin_window);
-  layout->bin_window = NULL;
+  gdk_window_set_user_data (priv->bin_window, NULL);
+  gdk_window_destroy (priv->bin_window);
+  priv->bin_window = NULL;
 
   GTK_WIDGET_CLASS (gtk_layout_parent_class)->unrealize (widget);
 }
@@ -943,23 +975,8 @@ static void
 gtk_layout_size_request (GtkWidget     *widget,
 			 GtkRequisition *requisition)
 {
-  GtkLayout *layout = GTK_LAYOUT (widget);
-  GList *tmp_list;
-
   requisition->width = 0;
   requisition->height = 0;
-
-  tmp_list = layout->children;
-
-  while (tmp_list)
-    {
-      GtkLayoutChild *child = tmp_list->data;
-      GtkRequisition child_requisition;
-      
-      tmp_list = tmp_list->next;
-
-      gtk_widget_size_request (child->widget, &child_requisition);
-    }
 }
 
 static void     
@@ -967,11 +984,12 @@ gtk_layout_size_allocate (GtkWidget     *widget,
 			  GtkAllocation *allocation)
 {
   GtkLayout *layout = GTK_LAYOUT (widget);
+  GtkLayoutPrivate *priv = layout->priv;
   GList *tmp_list;
 
-  widget->allocation = *allocation;
+  gtk_widget_set_allocation (widget, allocation);
 
-  tmp_list = layout->children;
+  tmp_list = priv->children;
 
   while (tmp_list)
     {
@@ -983,38 +1001,37 @@ gtk_layout_size_allocate (GtkWidget     *widget,
 
   if (gtk_widget_get_realized (widget))
     {
-      gdk_window_move_resize (widget->window,
+      gdk_window_move_resize (gtk_widget_get_window (widget),
 			      allocation->x, allocation->y,
 			      allocation->width, allocation->height);
 
-      gdk_window_resize (layout->bin_window,
-			 MAX (layout->width, allocation->width),
-			 MAX (layout->height, allocation->height));
+      gdk_window_resize (priv->bin_window,
+			 MAX (priv->width, allocation->width),
+			 MAX (priv->height, allocation->height));
     }
 
-  layout->hadjustment->page_size = allocation->width;
-  layout->hadjustment->page_increment = allocation->width * 0.9;
-  layout->hadjustment->lower = 0;
+  priv->hadjustment->page_size = allocation->width;
+  priv->hadjustment->page_increment = allocation->width * 0.9;
+  priv->hadjustment->lower = 0;
   /* set_adjustment_upper() emits ::changed */
-  gtk_layout_set_adjustment_upper (layout->hadjustment, MAX (allocation->width, layout->width), TRUE);
+  gtk_layout_set_adjustment_upper (priv->hadjustment, MAX (allocation->width, priv->width), TRUE);
 
-  layout->vadjustment->page_size = allocation->height;
-  layout->vadjustment->page_increment = allocation->height * 0.9;
-  layout->vadjustment->lower = 0;
-  layout->vadjustment->upper = MAX (allocation->height, layout->height);
-  gtk_layout_set_adjustment_upper (layout->vadjustment, MAX (allocation->height, layout->height), TRUE);
+  priv->vadjustment->page_size = allocation->height;
+  priv->vadjustment->page_increment = allocation->height * 0.9;
+  priv->vadjustment->lower = 0;
+  priv->vadjustment->upper = MAX (allocation->height, priv->height);
+  gtk_layout_set_adjustment_upper (priv->vadjustment, MAX (allocation->height, priv->height), TRUE);
 }
 
-static gint 
-gtk_layout_expose (GtkWidget      *widget,
-                   GdkEventExpose *event)
+static gboolean
+gtk_layout_draw (GtkWidget *widget,
+                 cairo_t   *cr)
 {
   GtkLayout *layout = GTK_LAYOUT (widget);
+  GtkLayoutPrivate *priv = layout->priv;
 
-  if (event->window != layout->bin_window)
-    return FALSE;
-
-  GTK_WIDGET_CLASS (gtk_layout_parent_class)->expose_event (widget, event);
+  if (gtk_cairo_should_draw_window (cr, priv->bin_window))
+    GTK_WIDGET_CLASS (gtk_layout_parent_class)->draw (widget, cr);
 
   return FALSE;
 }
@@ -1033,10 +1050,11 @@ gtk_layout_remove (GtkContainer *container,
 		   GtkWidget    *widget)
 {
   GtkLayout *layout = GTK_LAYOUT (container);
+  GtkLayoutPrivate *priv = layout->priv;
   GList *tmp_list;
   GtkLayoutChild *child = NULL;
 
-  tmp_list = layout->children;
+  tmp_list = priv->children;
   while (tmp_list)
     {
       child = tmp_list->data;
@@ -1049,7 +1067,7 @@ gtk_layout_remove (GtkContainer *container,
     {
       gtk_widget_unparent (widget);
 
-      layout->children = g_list_remove_link (layout->children, tmp_list);
+      priv->children = g_list_remove_link (priv->children, tmp_list);
       g_list_free_1 (tmp_list);
       g_free (child);
     }
@@ -1062,10 +1080,11 @@ gtk_layout_forall (GtkContainer *container,
 		   gpointer      callback_data)
 {
   GtkLayout *layout = GTK_LAYOUT (container);
+  GtkLayoutPrivate *priv = layout->priv;
   GtkLayoutChild *child;
   GList *tmp_list;
 
-  tmp_list = layout->children;
+  tmp_list = priv->children;
   while (tmp_list)
     {
       child = tmp_list->data;
@@ -1087,7 +1106,8 @@ gtk_layout_allocate_child (GtkLayout      *layout,
 
   allocation.x = child->x;
   allocation.y = child->y;
-  gtk_widget_get_child_requisition (child->widget, &requisition);
+
+  gtk_widget_get_preferred_size (child->widget, &requisition, NULL);
   allocation.width = requisition.width;
   allocation.height = requisition.height;
   
@@ -1100,18 +1120,17 @@ static void
 gtk_layout_adjustment_changed (GtkAdjustment *adjustment,
 			       GtkLayout     *layout)
 {
-  if (layout->freeze_count)
+  GtkLayoutPrivate *priv = layout->priv;
+
+  if (priv->freeze_count)
     return;
 
   if (gtk_widget_get_realized (GTK_WIDGET (layout)))
     {
-      gdk_window_move (layout->bin_window,
-		       - layout->hadjustment->value,
-		       - layout->vadjustment->value);
-      
-      gdk_window_process_updates (layout->bin_window, TRUE);
+      gdk_window_move (priv->bin_window,
+		       - priv->hadjustment->value,
+		       - priv->vadjustment->value);
+
+      gdk_window_process_updates (priv->bin_window, TRUE);
     }
 }
-
-#define __GTK_LAYOUT_C__
-#include "gtkaliasdef.c"
