@@ -22,8 +22,33 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
  */
 
+/**
+ * SECTION:gtklinkbutton
+ * @Title: GtkLinkButton
+ * @Short_description: Create buttons bound to a URL
+ * @See_also: #GtkButton
+ *
+ * A GtkLinkButton is a #GtkButton with a hyperlink, similar to the one
+ * used by web browsers, which triggers an action when clicked. It is useful
+ * to show quick links to resources.
+ *
+ * A link button is created by calling either gtk_link_button_new() or
+ * gtk_link_button_new_with_label(). If using the former, the URI you pass
+ * to the constructor is used as a label for the widget.
+ *
+ * The URI bound to a GtkLinkButton can be set specifically using
+ * gtk_link_button_set_uri(), and retrieved using gtk_link_button_get_uri().
+ *
+ * By default, GtkLinkButton calls gtk_show_uri() when the button is
+ * clicked. This behaviour can be overridden by connecting to the
+ * #GtkLinkButton::activate-link signal and returning %TRUE from the
+ * signal handler.
+ */
+
 #include "config.h"
 
+#include "gtklinkbutton.h"
+
 #include <string.h>
 
 #include "gtkclipboard.h"
@@ -31,14 +56,14 @@
 #include "gtkimagemenuitem.h"
 #include "gtklabel.h"
 #include "gtkmain.h"
+#include "gtkmarshalers.h"
 #include "gtkmenu.h"
 #include "gtkmenuitem.h"
+#include "gtksizerequest.h"
 #include "gtkstock.h"
 #include "gtkshow.h"
 #include "gtktooltip.h"
 
-#include "gtklinkbutton.h"
-
 #include "gtkintl.h"
 
 
@@ -58,6 +83,12 @@ enum
   PROP_VISITED
 };
 
+enum
+{
+  ACTIVATE_LINK,
+
+  LAST_SIGNAL
+};
 
 static void     gtk_link_button_finalize     (GObject          *object);
 static void     gtk_link_button_get_property (GObject          *object,
@@ -94,7 +125,7 @@ static gboolean gtk_link_button_query_tooltip_cb (GtkWidget    *widget,
                                                   gboolean      keyboard_tip,
                                                   GtkTooltip   *tooltip,
                                                   gpointer      data);
-
+static gboolean gtk_link_button_activate_link (GtkLinkButton *link_button);
 
 static const GtkTargetEntry link_drop_types[] = {
   { "text/uri-list", 0, 0 },
@@ -104,9 +135,7 @@ static const GtkTargetEntry link_drop_types[] = {
 static const GdkColor default_link_color = { 0, 0, 0, 0xeeee };
 static const GdkColor default_visited_link_color = { 0, 0x5555, 0x1a1a, 0x8b8b };
 
-static GtkLinkButtonUriFunc uri_func = NULL;
-static gpointer uri_func_data = NULL;
-static GDestroyNotify uri_func_destroy = NULL;
+static guint link_signals[LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GtkLinkButton, gtk_link_button, GTK_TYPE_BUTTON)
 
@@ -130,6 +159,8 @@ gtk_link_button_class_init (GtkLinkButtonClass *klass)
 
   button_class->clicked = gtk_link_button_clicked;
 
+  klass->activate_link = gtk_link_button_activate_link;
+
   /**
    * GtkLinkButton:uri
    * 
@@ -161,6 +192,29 @@ gtk_link_button_class_init (GtkLinkButtonClass *klass)
                                                          G_PARAM_READWRITE));
   
   g_type_class_add_private (gobject_class, sizeof (GtkLinkButtonPrivate));
+
+  /**
+   * GtkLinkButton::activate-link:
+   * @button: the #GtkLinkButton that emitted the signal
+   *
+   * The ::activate-link signal is emitted each time the #GtkLinkButton
+   * has been clicked.
+   *
+   * The default handler will call gtk_show_uri() with the URI stored inside
+   * the #GtkLinkButton:uri property.
+   *
+   * To override the default behavior, you can connect to the ::activate-link
+   * signal and stop the propagation of the signal by returning %TRUE from
+   * your handler.
+   */
+  link_signals[ACTIVATE_LINK] =
+    g_signal_new (I_("activate-link"),
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (GtkLinkButtonClass, activate_link),
+                  _gtk_boolean_handled_accumulator, NULL,
+                  _gtk_marshal_BOOLEAN__VOID,
+                  G_TYPE_BOOLEAN, 0);
 }
 
 static void
@@ -368,7 +422,7 @@ popup_position_func (GtkMenu  *menu,
 
   gdk_window_get_origin (gtk_widget_get_window (widget), x, y);
 
-  gtk_widget_size_request (priv->popup_menu, &req);
+  gtk_widget_get_preferred_size (priv->popup_menu, &req, NULL);
 
   gtk_widget_get_allocation (widget, &allocation);
   *x += allocation.width / 2;
@@ -470,35 +524,40 @@ gtk_link_button_button_press (GtkWidget      *widget,
   return FALSE;
 }
 
-static void
-gtk_link_button_clicked (GtkButton *button)
+static gboolean
+gtk_link_button_activate_link (GtkLinkButton *link_button)
 {
-  GtkLinkButton *link_button = GTK_LINK_BUTTON (button);
+  GdkScreen *screen;
+  GError *error;
 
-  if (uri_func)
-    (* uri_func) (link_button, link_button->priv->uri, uri_func_data);
+  if (gtk_widget_has_screen (GTK_WIDGET (link_button)))
+    screen = gtk_widget_get_screen (GTK_WIDGET (link_button));
   else
+    screen = NULL;
+
+  error = NULL;
+  gtk_show_uri (screen, link_button->priv->uri, GDK_CURRENT_TIME, &error);
+  if (error)
     {
-      GdkScreen *screen;
-      GError *error;
+      g_warning ("Unable to show '%s': %s",
+                 link_button->priv->uri,
+                 error->message);
+      g_error_free (error);
 
-      if (gtk_widget_has_screen (GTK_WIDGET (button)))
-        screen = gtk_widget_get_screen (GTK_WIDGET (button));
-      else
-        screen = NULL;
-
-      error = NULL;
-      gtk_show_uri (screen, link_button->priv->uri, GDK_CURRENT_TIME, &error);
-      if (error)
-        {
-          g_warning ("Unable to show '%s': %s",
-                     link_button->priv->uri,
-                     error->message);
-          g_error_free (error);
-        }
+      return FALSE;
     }
 
   gtk_link_button_set_visited (link_button, TRUE);
+
+  return TRUE;
+}
+
+static void
+gtk_link_button_clicked (GtkButton *button)
+{
+  gboolean retval = FALSE;
+
+  g_signal_emit (button, link_signals[ACTIVATE_LINK], 0, &retval);
 }
 
 static gboolean
@@ -703,41 +762,6 @@ gtk_link_button_get_uri (GtkLinkButton *link_button)
   return link_button->priv->uri;
 }
 
-/**
- * gtk_link_button_set_uri_hook:
- * @func: (allow-none): a function called each time a #GtkLinkButton is clicked, or %NULL
- * @data: (allow-none): user data to be passed to @func, or %NULL
- * @destroy: (allow-none): a #GDestroyNotify that gets called when @data is no longer needed, or %NULL
- *
- * Sets @func as the function that should be invoked every time a user clicks
- * a #GtkLinkButton. This function is called before every callback registered
- * for the "clicked" signal.
- *
- * If no uri hook has been set, GTK+ defaults to calling gtk_show_uri().
- *
- * Return value: the previously set hook function.
- *
- * Since: 2.10
- */
-GtkLinkButtonUriFunc
-gtk_link_button_set_uri_hook (GtkLinkButtonUriFunc func,
-			      gpointer             data,
-			      GDestroyNotify       destroy)
-{
-  GtkLinkButtonUriFunc old_uri_func;
-
-  if (uri_func_destroy)
-    (* uri_func_destroy) (uri_func_data);
-
-  old_uri_func = uri_func;
-
-  uri_func = func;
-  uri_func_data = data;
-  uri_func_destroy = destroy;
-
-  return old_uri_func;
-}
-
 /**
  * gtk_link_button_set_visited:
  * @link_button: a #GtkLinkButton
