@@ -78,6 +78,9 @@ static void gtk_table_size_request  (GtkWidget	    *widget,
 				     GtkRequisition *requisition);
 static void gtk_table_size_allocate (GtkWidget	    *widget,
 				     GtkAllocation  *allocation);
+static void gtk_table_compute_expand (GtkWidget     *widget,
+                                      gboolean      *hexpand,
+                                      gboolean      *vexpand);
 static void gtk_table_add	    (GtkContainer   *container,
 				     GtkWidget	    *widget);
 static void gtk_table_remove	    (GtkContainer   *container,
@@ -133,6 +136,7 @@ gtk_table_class_init (GtkTableClass *class)
   
   widget_class->size_request = gtk_table_size_request;
   widget_class->size_allocate = gtk_table_size_allocate;
+  widget_class->compute_expand = gtk_table_compute_expand;
   
   container_class->add = gtk_table_add;
   container_class->remove = gtk_table_remove;
@@ -140,7 +144,7 @@ gtk_table_class_init (GtkTableClass *class)
   container_class->child_type = gtk_table_child_type;
   container_class->set_child_property = gtk_table_set_child_property;
   container_class->get_child_property = gtk_table_get_child_property;
-  
+  gtk_container_class_handle_border_width (container_class);
 
   g_object_class_install_property (gobject_class,
                                    PROP_N_ROWS,
@@ -246,6 +250,43 @@ gtk_table_class_init (GtkTableClass *class)
   g_type_class_add_private (class, sizeof (GtkTablePrivate));
 }
 
+static void
+gtk_table_compute_expand (GtkWidget *widget,
+                          gboolean  *hexpand_p,
+                          gboolean  *vexpand_p)
+{
+  GtkTable *table = GTK_TABLE (widget);
+  GtkTablePrivate *priv = table->priv;
+  GList *list;
+  GtkTableChild *child;
+  gboolean hexpand;
+  gboolean vexpand;
+
+  hexpand = FALSE;
+  vexpand = FALSE;
+
+  for (list = priv->children; list; list = list->next)
+    {
+      child = list->data;
+
+      if (!hexpand)
+        hexpand = child->xexpand ||
+                  gtk_widget_compute_expand (child->widget,
+                                             GTK_ORIENTATION_HORIZONTAL);
+
+      if (!vexpand)
+        vexpand = child->yexpand ||
+                  gtk_widget_compute_expand (child->widget,
+                                             GTK_ORIENTATION_VERTICAL);
+
+      if (hexpand && vexpand)
+        break;
+    }
+
+  *hexpand_p = hexpand;
+  *vexpand_p = vexpand;
+}
+
 static GType
 gtk_table_child_type (GtkContainer   *container)
 {
@@ -373,14 +414,36 @@ gtk_table_set_child_property (GtkContainer    *container,
 	gtk_table_resize (table, table_child->bottom_attach, priv->ncols);
       break;
     case CHILD_PROP_X_OPTIONS:
-      table_child->xexpand = (g_value_get_flags (value) & GTK_EXPAND) != 0;
-      table_child->xshrink = (g_value_get_flags (value) & GTK_SHRINK) != 0;
-      table_child->xfill = (g_value_get_flags (value) & GTK_FILL) != 0;
+      {
+        gboolean xexpand;
+
+        xexpand = (g_value_get_flags (value) & GTK_EXPAND) != 0;
+
+        if (table_child->xexpand != xexpand)
+          {
+            table_child->xexpand = xexpand;
+            gtk_widget_queue_compute_expand (GTK_WIDGET (table));
+          }
+
+        table_child->xshrink = (g_value_get_flags (value) & GTK_SHRINK) != 0;
+        table_child->xfill = (g_value_get_flags (value) & GTK_FILL) != 0;
+      }
       break;
     case CHILD_PROP_Y_OPTIONS:
-      table_child->yexpand = (g_value_get_flags (value) & GTK_EXPAND) != 0;
-      table_child->yshrink = (g_value_get_flags (value) & GTK_SHRINK) != 0;
-      table_child->yfill = (g_value_get_flags (value) & GTK_FILL) != 0;
+      {
+        gboolean yexpand;
+
+        yexpand = (g_value_get_flags (value) & GTK_EXPAND) != 0;
+
+        if (table_child->yexpand != yexpand)
+          {
+            table_child->yexpand = yexpand;
+            gtk_widget_queue_compute_expand (GTK_WIDGET (table));
+          }
+
+        table_child->yshrink = (g_value_get_flags (value) & GTK_SHRINK) != 0;
+        table_child->yfill = (g_value_get_flags (value) & GTK_FILL) != 0;
+      }
       break;
     case CHILD_PROP_X_PADDING:
       table_child->xpadding = g_value_get_uint (value);
@@ -912,7 +975,6 @@ gtk_table_size_request (GtkWidget      *widget,
   GtkTable *table = GTK_TABLE (widget);
   GtkTablePrivate *priv = table->priv;
   gint row, col;
-  guint border_width;
 
   requisition->width = 0;
   requisition->height = 0;
@@ -932,10 +994,6 @@ gtk_table_size_request (GtkWidget      *widget,
     requisition->height += priv->rows[row].requisition;
   for (row = 0; row + 1 < priv->nrows; row++)
     requisition->height += priv->rows[row].spacing;
-
-  border_width = gtk_container_get_border_width (GTK_CONTAINER (table));
-  requisition->width += border_width * 2;
-  requisition->height += border_width * 2;
 }
 
 static void
@@ -1037,14 +1095,13 @@ gtk_table_size_request_init (GtkTable *table)
     {
       child = children->data;
       children = children->next;
-      
-      if (gtk_widget_get_visible (child->widget))
-        gtk_widget_get_preferred_size (child->widget, NULL, NULL);
 
-      if (child->left_attach == (child->right_attach - 1) && child->xexpand)
+      if (child->left_attach == (child->right_attach - 1) &&
+          (child->xexpand || gtk_widget_compute_expand (child->widget, GTK_ORIENTATION_HORIZONTAL)))
 	priv->cols[child->left_attach].expand = TRUE;
       
-      if (child->top_attach == (child->bottom_attach - 1) && child->yexpand)
+      if (child->top_attach == (child->bottom_attach - 1) &&
+          (child->yexpand || gtk_widget_compute_expand (child->widget, GTK_ORIENTATION_VERTICAL)))
 	priv->rows[child->top_attach].expand = TRUE;
     }
 }
@@ -1290,7 +1347,7 @@ gtk_table_size_allocate_init (GtkTable *table)
 	{
 	  if (child->left_attach == (child->right_attach - 1))
 	    {
-	      if (child->xexpand)
+	      if (child->xexpand || gtk_widget_compute_expand (child->widget, GTK_ORIENTATION_HORIZONTAL))
 		priv->cols[child->left_attach].expand = TRUE;
 
 	      if (!child->xshrink)
@@ -1301,7 +1358,7 @@ gtk_table_size_allocate_init (GtkTable *table)
 	  
 	  if (child->top_attach == (child->bottom_attach - 1))
 	    {
-	      if (child->yexpand)
+	      if (child->yexpand || gtk_widget_compute_expand (child->widget, GTK_ORIENTATION_VERTICAL))
 		priv->rows[child->top_attach].expand = TRUE;
 	      
 	      if (!child->yshrink)
@@ -1448,16 +1505,14 @@ gtk_table_size_allocate_pass1 (GtkTable *table)
   gint nexpand;
   gint nshrink;
   gint extra;
-  guint border_width;
 
   /* If we were allocated more space than we requested
    *  then we have to expand any expandable rows and columns
    *  to fill in the extra space.
    */
   gtk_widget_get_allocation (GTK_WIDGET (table), &allocation);
-  border_width = gtk_container_get_border_width (GTK_CONTAINER (table));
-  real_width = allocation.width - border_width * 2;
-  real_height = allocation.height - border_width * 2;
+  real_width = allocation.width;
+  real_height = allocation.height;
 
   if (priv->homogeneous)
     {
@@ -1666,15 +1721,13 @@ gtk_table_size_allocate_pass2 (GtkTable *table)
       if (gtk_widget_get_visible (child->widget))
 	{
 	  GtkRequisition child_requisition;
-          guint border_width;
 
           gtk_widget_get_preferred_size (child->widget,
                                          &child_requisition, NULL);
 
           gtk_widget_get_allocation (GTK_WIDGET (table), &table_allocation);
-          border_width = gtk_container_get_border_width (GTK_CONTAINER (table));
-	  x = table_allocation.x + border_width;
-	  y = table_allocation.y + border_width;
+	  x = table_allocation.x;
+	  y = table_allocation.y;
 	  max_width = 0;
 	  max_height = 0;
 	  
