@@ -12,6 +12,10 @@
 
 G_BEGIN_DECLS
 
+/* enumerations from "gtkaboutdialog.h" */
+GType gtk_license_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_LICENSE (gtk_license_get_type ())
+
 /* enumerations from "gtkaccelgroup.h" */
 GType gtk_accel_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_ACCEL_FLAGS (gtk_accel_flags_get_type ())
@@ -59,8 +63,8 @@ GType gtk_entry_icon_position_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_ENTRY_ICON_POSITION (gtk_entry_icon_position_get_type ())
 
 /* enumerations from "gtkenums.h" */
-GType gtk_anchor_type_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_ANCHOR_TYPE (gtk_anchor_type_get_type ())
+GType gtk_align_get_type (void) G_GNUC_CONST;
+#define GTK_TYPE_ALIGN (gtk_align_get_type ())
 GType gtk_arrow_placement_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_ARROW_PLACEMENT (gtk_arrow_placement_get_type ())
 GType gtk_arrow_type_get_type (void) G_GNUC_CONST;
@@ -119,16 +123,10 @@ GType gtk_shadow_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_SHADOW_TYPE (gtk_shadow_type_get_type ())
 GType gtk_state_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_STATE_TYPE (gtk_state_type_get_type ())
-GType gtk_submenu_direction_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_SUBMENU_DIRECTION (gtk_submenu_direction_get_type ())
-GType gtk_submenu_placement_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_SUBMENU_PLACEMENT (gtk_submenu_placement_get_type ())
 GType gtk_toolbar_style_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_TOOLBAR_STYLE (gtk_toolbar_style_get_type ())
 GType gtk_update_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_UPDATE_TYPE (gtk_update_type_get_type ())
-GType gtk_visibility_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_VISIBILITY (gtk_visibility_get_type ())
 GType gtk_window_position_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_WINDOW_POSITION (gtk_window_position_get_type ())
 GType gtk_window_type_get_type (void) G_GNUC_CONST;
@@ -198,10 +196,6 @@ GType gtk_buttons_type_get_type (void) G_GNUC_CONST;
 GType gtk_notebook_tab_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_NOTEBOOK_TAB (gtk_notebook_tab_get_type ())
 
-/* enumerations from "gtkobject.h" */
-GType gtk_object_flags_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_OBJECT_FLAGS (gtk_object_flags_get_type ())
-
 /* enumerations from "gtkprintoperation.h" */
 GType gtk_print_status_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PRINT_STATUS (gtk_print_status_get_type ())
@@ -212,14 +206,6 @@ GType gtk_print_operation_action_get_type (void) G_GNUC_CONST;
 GType gtk_print_error_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PRINT_ERROR (gtk_print_error_get_type ())
 
-/* enumerations from "gtkprivate.h" */
-GType gtk_private_flags_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_PRIVATE_FLAGS (gtk_private_flags_get_type ())
-
-/* enumerations from "gtkprogressbar.h" */
-GType gtk_progress_bar_orientation_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_PROGRESS_BAR_ORIENTATION (gtk_progress_bar_orientation_get_type ())
-
 /* enumerations from "gtkrc.h" */
 GType gtk_rc_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_RC_FLAGS (gtk_rc_flags_get_type ())
@@ -287,8 +273,6 @@ GType gtk_ui_manager_item_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_UI_MANAGER_ITEM_TYPE (gtk_ui_manager_item_type_get_type ())
 
 /* enumerations from "gtkwidget.h" */
-GType gtk_widget_flags_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_WIDGET_FLAGS (gtk_widget_flags_get_type ())
 GType gtk_widget_help_type_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_WIDGET_HELP_TYPE (gtk_widget_help_type_get_type ())
 G_END_DECLS
