@@ -1,4 +1,3 @@
-#undef GDK_DISABLE_DEPRECATED
 #include <gtk/gtk.h>
 #ifdef GDK_WINDOWING_X11
 #include <X11/Xlib.h>
@@ -12,13 +11,6 @@ static void update_store (void);
 
 static GtkWidget *main_window;
 
-static gboolean
-window_has_impl (GdkWindow *window)
-{
-  GdkWindowObject *w;
-  w = (GdkWindowObject *)window;
-  return w->parent == NULL || w->parent->impl != w->impl;
-}
 
 GdkWindow *
 create_window (GdkWindow *parent,
@@ -59,7 +51,6 @@ create_window (GdkWindow *parent,
       bg->green = g_random_int_range (0, 0xffff);;
     }
   
-  gdk_rgb_find_color (gtk_widget_get_colormap (darea), bg);
   gdk_window_set_background (window, bg);
   g_object_set_data_full (G_OBJECT (window), "color", bg, g_free);
   
@@ -216,7 +207,7 @@ add_window_clicked (GtkWidget *button,
   if (l != NULL)
     parent = l->data;
   else
-    parent = darea->window;
+    parent = gtk_widget_get_window (darea);
 
   g_list_free (l);
   
@@ -224,22 +215,6 @@ add_window_clicked (GtkWidget *button,
   update_store ();
 }
 
-static void
-draw_drawable_clicked (GtkWidget *button, 
-		       gpointer data)
-{
-  GdkGC *gc;
-  gc = gdk_gc_new (darea->window);
-  gdk_draw_drawable (darea->window,
-		     gc,
-		     darea->window,
-		     -15, -15,
-		     40, 70,
-		     100, 100);
-  g_object_unref (gc);
-}
-
-
 static void
 remove_window_clicked (GtkWidget *button, 
 		       gpointer data)
@@ -262,17 +237,18 @@ static void
 save_window (GString *s,
 	     GdkWindow *window)
 {
-  gint x, y, w, h;
+  gint x, y;
   GdkColor *color;
 
   gdk_window_get_position (window, &x, &y);
-  gdk_drawable_get_size (GDK_DRAWABLE (window), &w, &h);
   color = g_object_get_data (G_OBJECT (window), "color");
   
   g_string_append_printf (s, "%d,%d %dx%d (%d,%d,%d) %d %d\n",
-			  x, y, w, h,
+			  x, y,
+                          gdk_window_get_width (window),
+                          gdk_window_get_height (window),
 			  color->red, color->green, color->blue,
-			  window_has_impl (window),
+			  gdk_window_has_native (window),
 			  g_list_length (gdk_window_peek_children (window)));
 
   save_children (s, window);
@@ -307,7 +283,7 @@ save_clicked (GtkWidget *button,
 
   s = g_string_new ("");
 
-  save_children (s, darea->window);
+  save_children (s, gtk_widget_get_window (darea));
 
   dialog = gtk_file_chooser_dialog_new ("Filename for window data",
 					NULL,
@@ -385,18 +361,21 @@ parse_window (GdkWindow *parent, char **lines)
 static void
 load_file (GFile *file)
 {
+  GdkWindow *window;
   char *data;
   char **lines, **l;
   
   if (g_file_load_contents (file, NULL, &data, NULL, NULL, NULL))
     {
-      destroy_children (darea->window);
+      window = gtk_widget_get_window (darea);
+
+      destroy_children (window);
 
       lines = g_strsplit (data, "\n", -1);
 
       l = lines;
       while (*l != NULL)
-	l = parse_window (darea->window, l);
+	l = parse_window (window, l);
     }
 
   update_store ();
@@ -460,7 +439,8 @@ manual_clicked (GtkWidget *button,
     return;
 
   gdk_window_get_position (selected->data, &x, &y);
-  gdk_drawable_get_size (selected->data, &w, &h);
+  w = gdk_window_get_width (selected->data);
+  h = gdk_window_get_height (selected->data);
 
   dialog = gtk_dialog_new_with_buttons ("Select new position and size",
 					GTK_WINDOW (main_window),
@@ -664,10 +644,8 @@ smaller_window_clicked (GtkWidget *button,
     {
       window = l->data;
       
-      gdk_drawable_get_size (GDK_DRAWABLE (window), &w, &h);
-      
-      w -= 10;
-      h -= 10;
+      w = gdk_window_get_width (window) - 10;
+      h = gdk_window_get_height (window) - 10;
       if (w < 1)
 	w = 1;
       if (h < 1)
@@ -693,10 +671,8 @@ larger_window_clicked (GtkWidget *button,
     {
       window = l->data;
       
-      gdk_drawable_get_size (GDK_DRAWABLE (window), &w, &h);
-      
-      w += 10;
-      h += 10;
+      w = gdk_window_get_width (window) + 10;
+      h = gdk_window_get_height (window) + 10;
       
       gdk_window_resize (window, w, h);
     }
@@ -757,14 +733,14 @@ render_window_cell (GtkTreeViewColumn *tree_column,
 		      0, &window,
 		      -1);
 
-  if (window_has_impl (window))
+  if (gdk_window_has_native (window))
       name = g_strdup_printf ("%p (native)", window);
   else
       name = g_strdup_printf ("%p", window);
+
   g_object_set (cell,
 		"text", name,
-		"background-gdk", &((GdkWindowObject *)window)->bg_color,
-		NULL);  
+		NULL);
 }
 
 static void
@@ -797,7 +773,7 @@ update_store (void)
 
   gtk_tree_store_clear (window_store);
 
-  add_children (window_store, darea->window, NULL);
+  add_children (window_store, gtk_widget_get_window (darea), NULL);
   gtk_tree_view_expand_all (GTK_TREE_VIEW (treeview));
 
   select_windows (selected);
@@ -1051,16 +1027,6 @@ main (int argc, char **argv)
 			     3, 4);
   gtk_widget_show (button);
 
-  button = gtk_button_new_with_label ("draw drawable");
-  gtk_box_pack_start (GTK_BOX (vbox),
-		      button,
-		      FALSE, FALSE,
-		      2);
-  gtk_widget_show (button);
-  g_signal_connect (button, "clicked", 
-		    G_CALLBACK (draw_drawable_clicked), 
-		    NULL);
-
   button = gtk_button_new_with_label ("Add window");
   gtk_box_pack_start (GTK_BOX (vbox),
 		      button,
